﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace SmartCowBioControl.src.calcul
{
    /// <summary>
    /// Classe pour le calcul des indices.
    /// </summary>
    public static class CalculIndices
    {
        // ====================================================================================================
        // METHODES PUBLIQUES
        // ====================================================================================================

        /// <summary>
        /// Calcul de l'indice de Jaccard.
        /// </summary>
        /// <param name="liste1"></param>
        /// <param name="liste2"></param>
        /// <returns></returns>
        public static double IndiceJaccard(Dictionary<DateTime, bool> donnees1, Dictionary<DateTime, bool> donnees2)
        {
            if (donnees1 == null ||
                donnees2 == null ||
                donnees1.Count == 0 ||
                donnees2.Count == 0)
            {
                return 0;
            }

            int nbValeurs = 0;
            double m11 = 0;
            double m00 = 0;

            foreach (var donnee1 in donnees1)
            {
                if (donnees2.ContainsKey(donnee1.Key))
                {
                    nbValeurs++;

                    if (donnee1.Value && donnees2[donnee1.Key])
                    {
                        m11++;
                    }
                    else if (!donnee1.Value && !donnees2[donnee1.Key])
                    {
                        m00++;
                    }
                }
            }

            return nbValeurs == 0 ? 0 : m11 / (nbValeurs - m00);
        }

        /// <summary>
        /// Calcul du pourcentage de true.
        /// </summary>
        /// <param name="liste"></param>
        /// <returns></returns>
        public static double PourcentageTrue(List<bool> liste)
        {
            return liste == null || liste.Count == 0 ? 0 : liste.Count(l => l) / (double)liste.Count;
        }

        /// <summary>
        /// Calcul de la fréquence à true en minutes.
        /// </summary>
        /// <param name="liste"></param>
        /// <param name="pasDeTempsSecondes"></param>
        /// <returns></returns>
        public static double FrequenceTrueMinutes(List<bool> liste, int pasDeTempsSecondes)
        {
            if (liste == null ||
                liste.Count == 0)
            {
                return 0;
            }

            int nbPassageATrue = 0;

            bool valeurPrecedente = false;
            foreach (bool valeur in liste)
            {
                if (!valeurPrecedente && valeur)
                {
                    nbPassageATrue++;
                }

                valeurPrecedente = valeur;
            }

            TimeSpan duree = TimeSpan.FromSeconds(liste.Count * pasDeTempsSecondes);

            return nbPassageATrue / duree.TotalMinutes;
        }
    }
}
