﻿using SmartCowBioControl.src.objects;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms.DataVisualization.Charting;

namespace SmartCowBioControl.src.form
{
    /// <summary>
    /// Fenêtre principale.
    /// Partie de classe pour les graphes.
    /// </summary>
    public partial class FenetrePrincipale
    {
        // ====================================================================================================
        // METHODES PRIVEES
        // ====================================================================================================

        /// <summary>
        /// Dessiner le graphique.
        /// </summary>
        /// <param name="donneesVideo"></param>
        /// <param name="rapids"></param>
        /// <param name="minDate"></param>
        /// <param name="maxDate"></param>
        /// <param name="minPoids"></param>
        /// <param name="maxPoids"></param>
        private void DessinerGraphique(
            Dictionary<DateTime, int> donneesVideo,
            List<RapidObject> rapidSansFiltres,
            List<RapidObject> rapidsAvecFiltres,
            Dictionary<DateTime, int> donneesSeuil,
            DateTime minDate,
            DateTime maxDate,
            double minPoidsSansFiltres,
            double maxPoidsSansFiltres,
            double minPoidsAvecFiltres,
            double maxPoidsAvecFiltres)
        {
            ChartArea chartAreaBrut = this.chart.ChartAreas[0];
            ChartArea chartAreaFiltre = this.chart.ChartAreas[1];

            Series serieRapidSansFiltres = this.chart.Series[0];
            Series serieVideoBrut = this.chart.Series[1];
            Series serieRapidAvecFiltres = this.chart.Series[2];
            Series serieVideoFiltre = this.chart.Series[3];
            Series serieSeuil = this.chart.Series[4];

            // --------------------------------------------------------------------------------
            // DONNEES BRUTES

            chartAreaBrut.AxisX.Interval = 1;
            chartAreaBrut.AxisX.IntervalOffset = 1;
            chartAreaBrut.AxisX.IntervalType = DateTimeIntervalType.Minutes;
            chartAreaBrut.AxisX.LabelStyle.Angle = -45;
            chartAreaBrut.AxisX.LabelStyle.Format = "HH:mm:ss";
            chartAreaBrut.AxisX.MajorGrid.LineColor = Color.DarkGray;
            chartAreaBrut.AxisX.Maximum = maxDate.ToOADate();
            chartAreaBrut.AxisX.Minimum = minDate.ToOADate();

            chartAreaBrut.AxisY.IsStartedFromZero = false;
            chartAreaBrut.AxisY.MajorGrid.LineColor = Color.DarkGray;
            chartAreaBrut.AxisY.Maximum = maxPoidsSansFiltres;
            chartAreaBrut.AxisY.Minimum = minPoidsSansFiltres;

            // Courbe Rapid sans filtres
            serieRapidSansFiltres.Points.Clear();
            if (rapidSansFiltres != null)
            {
                foreach (RapidObject rapid in rapidSansFiltres)
                {
                    serieRapidSansFiltres.Points.AddXY(rapid.DateHeureDephasage, rapid.Poids);
                }
            }

            // Courbe vidéo brut
            serieVideoBrut.Points.Clear();
            if (donneesVideo != null)
            {
                foreach (var key in donneesVideo)
                {
                    serieVideoBrut.Points.AddXY(key.Key, key.Value);
                }

                foreach (var pt in serieVideoBrut.Points)
                {
                    pt.YValues[0] = pt.YValues[0] == 1 ? maxPoidsSansFiltres : minPoidsSansFiltres;
                }
            }

            // --------------------------------------------------------------------------------
            // DONNEES FILTRES

            chartAreaFiltre.AxisX.Interval = 1;
            chartAreaFiltre.AxisX.IntervalOffset = 1;
            chartAreaFiltre.AxisX.IntervalType = DateTimeIntervalType.Minutes;
            chartAreaFiltre.AxisX.LabelStyle.Angle = -45;
            chartAreaFiltre.AxisX.LabelStyle.Format = "HH:mm:ss";
            chartAreaFiltre.AxisX.MajorGrid.LineColor = Color.DarkGray;
            chartAreaFiltre.AxisX.Maximum = maxDate.ToOADate();
            chartAreaFiltre.AxisX.Minimum = minDate.ToOADate();

            chartAreaFiltre.AxisY.InterlacedColor = Color.DarkGray;
            chartAreaFiltre.AxisY.IsStartedFromZero = false;
            chartAreaFiltre.AxisY.MajorGrid.LineColor = Color.DarkGray;
            chartAreaFiltre.AxisY.Maximum = maxPoidsAvecFiltres;
            chartAreaFiltre.AxisY.Minimum = this.parametres.IsSeuilActif ? -1 : minPoidsAvecFiltres;

            // Courbe Rapid avec filtres
            serieRapidAvecFiltres.Points.Clear();
            if (rapidsAvecFiltres != null)
            {
                foreach (var rapid in rapidsAvecFiltres)
                {
                    serieRapidAvecFiltres.Points.AddXY(rapid.DateHeureDephasage, rapid.Poids);
                }
            }

            // Courbe vidéo filtre
            serieVideoFiltre.Points.Clear();
            if (donneesVideo != null)
            {
                foreach (var key in donneesVideo)
                {
                    serieVideoFiltre.Points.AddXY(key.Key, key.Value);
                }

                foreach (var pt in serieVideoFiltre.Points)
                {
                    pt.YValues[0] = pt.YValues[0] == 1 ? maxPoidsAvecFiltres : minPoidsAvecFiltres;
                }
            }

            // Courbe seuil
            serieSeuil.Points.Clear();
            if (this.parametres.IsSeuilActif && donneesSeuil != null)
            {
                foreach (var valeurSeuil in donneesSeuil)
                {
                    serieSeuil.Points.AddXY(valeurSeuil.Key, valeurSeuil.Value);
                }
            }

            // --------------------------------------------------------------------------------

            chartAreaFiltre.AlignWithChartArea = "ChartArea1";

            this.chart.Invalidate();
        }
    }
}
