﻿using System.Windows.Forms;

namespace SmartCowBioControl.src.util
{
    /// <summary>
    /// Classe pour l'affichage des messages.
    /// </summary>
    public class AfficheurMessage
    {
        // ====================================================================================================
        // SINGLETON
        // ====================================================================================================

        /// <summary>
        /// Constructeur privé
        /// </summary>
        private AfficheurMessage() { }

        /// <summary>
        /// Holder
        /// </summary>
        private static class SingletonHolder
        {
            /// <summary>
            /// Instance unique non préinitialisée
            /// </summary>
            public static AfficheurMessage instance = new AfficheurMessage();
        }

        /// <summary>
        /// Point d'accès pour l'instance unique du singleton
        /// </summary>
        public static AfficheurMessage Instance { get { return SingletonHolder.instance; } }

        // ====================================================================================================
        // METHODES PUBLIQUES
        // ====================================================================================================

        /// <summary>
        /// Afficher un message d'erreur.
        /// </summary>
        /// <param name="message">un message d'erreur</param>
        public void AfficherMessageErreur(string message)
        {
            MessageBox.Show(message, "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        /// <summary>
        /// Afficher un message d'information.
        /// </summary>
        /// <param name="message">un message d'information</param>
        public void AfficherMessageInformation(string message)
        {
            MessageBox.Show(message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        /// <summary>
        /// Afficher un message d'avertissement.
        /// </summary>
        /// <param name="message">un message d'avertissement</param>
        public void AfficherMessageAvertissement(string message)
        {
            MessageBox.Show(message, "Avertissement", MessageBoxButtons.OK, MessageBoxIcon.Warning);
        }

        /// <summary>
        /// Afficher un message d'avertissement avec choix.
        /// </summary>
        /// <param name="message">un message d'erreur</param>
        public DialogResult AfficherMessageAvertissementChoix(string message)
        {
            return MessageBox.Show(message, "Avertissement", MessageBoxButtons.OKCancel, MessageBoxIcon.Warning);
        }
    }
}
