﻿using SmartCowBioControl.src.objects;
using System;
using System.Collections.Generic;
using System.Linq;

namespace SmartCowBioControl.src.calcul
{
    public static class CalculFiltres
    {
        /// <summary>
        /// Filtre haute fréquence.
        /// </summary>
        /// <param name="rapids"></param>
        /// <param name="paramX"></param>
        /// <param name="pasDeTemps"></param>
        public static void FiltreHauteFrequence(List<RapidObject> rapids, double paramX, int pasDeTemps)
        {
            double xn = Math.Floor(paramX / 2);
            xn = xn / pasDeTemps;

            //liste qui va correspondre aux points de la plage pour faire la moyenne
            List<double> plageList = new List<double>();
            //liste contenant les résultats des calculs
            List<double> resultatList = new List<double>();

            //parcours pour chaque point
            for (int i = 0; i < rapids.Count; i++)
            {
                //ajouter le i courant 
                plageList.Add(rapids[i].Poids);

                //parcours des Xn secondes avant et après i
                for (int j = 0; j < xn; j++)
                {
                    int ijNeg = i - j - 1;
                    int ijPos = i + j + 1;

                    if (ijNeg >= 0)
                    {
                        plageList.Add(rapids[ijNeg].Poids);
                    }

                    if (ijPos < rapids.Count)
                    {
                        plageList.Add(rapids[ijPos].Poids);
                    }
                }

                //calcul moyenne
                double moyenne = plageList.Average();

                resultatList.Add(moyenne);

                //vider la liste vu que la plage sera différente pour la prochaine itération
                plageList.Clear();
            }

            // remettre les résultats dans la liste rapids à retourner
            for (int i = 0; i < resultatList.Count; i++)
            {
                rapids[i].Poids = resultatList[i];
            }
        }

        /// <summary>
        /// Filtre basse fréquence.
        /// </summary>
        /// <param name="rapids"></param>
        /// <param name="seconds"></param>
        /// <param name="pasDeTemps"></param>
        public static void FiltreBasseFrequence(List<RapidObject> rapids, int seconds, int pasDeTemps)
        {
            if (seconds == 0)
            {
                seconds = 1;
            }
            if (pasDeTemps == 0)
            {
                pasDeTemps = 1;
            }
            seconds = seconds / pasDeTemps;

            //liste qui va correspondre aux points entre le poind de la première boucle for et les prochaines valeurs jusq'à X secondes
            List<double> tempList = new List<double>();
            double min = 0;

            //parcours pour chaque point
            for (int i = 0; i < rapids.Count; i++)
            {
                //parcours des X secondes 
                for (int j = 0; j < seconds; j++)
                {
                    int ij = i + j;

                    //cas ou on dépasse la capacité de la liste
                    if (ij >= rapids.Count)
                    {
                        break;
                    }

                    tempList.Add(rapids[ij].Poids);
                }

                //récupération du minimum
                if (tempList.Count > 0)
                {
                    min = tempList.Min();
                }
                tempList.Clear();

                //retranchement
                rapids[i].Poids = rapids[i].Poids - min;

                min = 0;
            }
        }

        /// <summary>
        /// Normalisation.
        /// </summary>
        /// <param name="rapids"></param>
        public static void Normalisation(List<RapidObject> rapids)
        {
            double max = rapids.Max(a => a.Poids);
            rapids.ForEach(r => r.Poids = r.Poids / max);
        }

        /// <summary>
        /// Seuil.
        /// </summary>
        /// <param name="rapids"></param>
        /// <param name="fenetreCalculMaxLocaux"></param>
        /// <param name="nbDecimales"></param>
        /// <param name="seuilMin"></param>
        /// <param name="seuilMax"></param>
        /// <param name="maxLocaux"></param>
        /// <param name="seuil"></param>
        /// <returns></returns>
        public static Dictionary<DateTime, int> Seuil(List<RapidObject> rapids, int fenetreCalculMaxLocaux, int nbDecimales, double seuilMin, double seuilMax, out List<RapidObject> maxLocaux, out double seuil)
        {
            // ----------------------------------------------------------------------------------------------------
            // Max locaux

            maxLocaux = new List<RapidObject>();

            for (int i = fenetreCalculMaxLocaux; i < rapids.Count - fenetreCalculMaxLocaux; i++)
            {
                if (Math.Round(rapids[i - fenetreCalculMaxLocaux].Poids, nbDecimales) < Math.Round(rapids[i].Poids, nbDecimales) &&
                    Math.Round(rapids[i + fenetreCalculMaxLocaux].Poids, nbDecimales) < Math.Round(rapids[i].Poids, nbDecimales))
                {
                    maxLocaux.Add(rapids[i]);
                    i += fenetreCalculMaxLocaux;
                }
            }

            // ----------------------------------------------------------------------------------------------------
            // Seuil

            seuil = maxLocaux.Count == 0 ? 0 : maxLocaux.Min(r => Math.Round(r.Poids, nbDecimales));
            seuil = seuil < seuilMin ? seuilMin : seuil;
            seuil = seuil > seuilMax ? seuilMax : seuil;

            double seuilCopie = seuil; // copie car interdit d'utiliser un paramètre out dans une expression lambda
            return rapids.ToDictionary(r => r.DateHeureDephasage, r => r.Poids < seuilCopie ? 0 : -1);
        }
    }
}
