﻿using ClosedXML.Excel;
using System;
using System.Data.SQLite;

namespace SmartCowBioControl.src.database
{
    /// <summary>
    /// Génération de la base de donnée des données rapides, récupération des données rapids selon un intervalle de temps 
    /// La fonction SelectDateTimeInterval est la seule qui sert pour le programme
    /// </summary>
    public static class RapidDatabase
    {
        /// <summary>
        /// La variable dt qui vaut 2019/06/10 reste fixe, car il y a un problème dans le fichier excel des fichiers rapides, après la ligne 48025,
        /// la date a un problème et il manque un "/" donc on ne peut pas parser le string en datetime.
        /// étant donné que les données du fichier rapide globalement sur un jour, ça ne pose pas problème, mais dans le cas contraire il faut relier dt à la colone O du fichier excel
        /// </summary>
        /// <param name="excelPath"></param>
        /// <param name="auge"></param>
        public static void SetFileDataInDb(string excelPath, int auge)
        {
            SQLiteConnection connection = new SQLiteConnection("Data Source=rapiddb.db");
            connection.Open();
            SQLiteCommand command = new SQLiteCommand(connection);
            var wb = new XLWorkbook(excelPath);
            IXLWorksheet wks = wb.Worksheet("Rapid");
            var usedRows = wks.RowsUsed();

            //Date simplifiée car les dates dans l'excel ont un problème
            DateTime dt = new DateTime(2019, 06, 10);

            foreach (var dataRow in usedRows)
            {
                if (dataRow.RowNumber() > 1)
                {
                    TimeSpan Ts = dataRow.Cell(2).GetTimeSpan();

                    command.CommandText = $"INSERT INTO 'Auge {auge}' (poid, date, animal) VALUES (@poid, @date, @animal)";
                    command.Parameters.Add(new SQLiteParameter("@poid", Int32.Parse(dataRow.Cell(5).GetString())));
                    command.Parameters.AddWithValue("@date", dt + Ts);
                    command.Parameters.Add(new SQLiteParameter("@animal", Int32.Parse(dataRow.Cell(4).GetString())));
                    command.ExecuteNonQuery();
                }
            }
        }

        /// <summary>
        /// Fonction pour générer toutes les tables de la base de données sur SQLITE
        /// </summary>
        public static void GenerateAllTable()
        {
            SQLiteConnection connection = new SQLiteConnection("Data Source=rapiddb.db");
            connection.Open();
            SQLiteCommand command = new SQLiteCommand(connection);
            for (int i = 15; i <= 26; i++)
            {
                command.CommandText = $"CREATE TABLE 'Auge {i}'('id'	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE,	'poid'	INTEGER NOT NULL,'date' INTEGER NOT NULL,'animal' INTEGER NOT NULL);";
                command.ExecuteNonQuery();

            }
        }
    }
}
