﻿using BrightIdeasSoftware;
using SmartCowBioControl.src.calcul;
using SmartCowBioControl.src.export;
using SmartCowBioControl.src.objects;
using SmartCowBioControl.src.objects.objects;
using SmartCowBioControl.src.services;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SmartCowBioControl.src.form
{
    /// <summary>
    /// Fenêtre principale.
    /// </summary>
    public partial class FenetrePrincipale : Form
    {
        // ====================================================================================================
        // ATTRIBUTS
        // ====================================================================================================

        private Parametres parametres = new Parametres();

        private ToolTip tooltipCalcul = new ToolTip();
        private ToolTip aideCocheVerte = new ToolTip();

        private List<Visite> visites;

        // ====================================================================================================
        // CONSTRUCTEURS
        // ====================================================================================================

        /// <summary>
        /// Constructeur.
        /// </summary>
        public FenetrePrincipale()
        {
            InitializeComponent();

            tooltipCalcul.InitialDelay = 0;
            tooltipCalcul.ShowAlways = true;

            //cacher les fleches verticales des NumericUpDown de l'interface
            visiteDephasageValeur.Controls[0].Visible = false;
            //SeuilNumeric.Controls[0].Visible = false;
            visiteIntervalleValeur.Controls[0].Visible = false;
            visiteDecalageValeur.Controls[0].Visible = false;

            //Numero version
            this.Text = this.Text + "      " + Application.ProductVersion.Substring(0, 3);

            this.parametresCheckBoxPasDeTemps.Checked = this.parametres.IsPasDeTempsActif;
            this.parametresCheckBoxFiltreHauteFrequence.Checked = this.parametres.IsFiltreHauteFrequenceActif;
            this.parametresCheckBoxFiltreBasseFrequence.Checked = this.parametres.IsFiltreBasseFrequenceActif;
            this.parametresCheckBoxNormalisation.Checked = this.parametres.IsNormalisationActif;
            this.parametresCheckBoxSeuil.Checked = this.parametres.IsSeuilActif;

            this.parametresTextBoxPasDeTemps.Text = this.parametres.PasDeTemps.ToString();
            this.parametresTextBoxFiltreHauteFrequence.Text = this.parametres.FiltreHauteFrequence.ToString();
            this.parametresTextBoxFiltreBasseFrequence.Text = this.parametres.FiltreBasseFrequence.ToString();
            this.parametresTextBoxSeuilNbDecimales.Text = this.parametres.SeuilNbDecimales.ToString();
            this.parametresTextBoxSeuilMin.Text = this.parametres.SeuilMin.ToString();
            this.parametresTextBoxSeuilMax.Text = this.parametres.SeuilMax.ToString();
            this.parametresTextBoxSeuilFenetreCalculMaxLocaux.Text = this.parametres.FenetreCalculMaxLocaux.ToString();

            this.visiteIntervalleValeur.Value = this.parametres.IntervalleDeTemps;
            this.visiteDecalageValeur.Value = this.parametres.DecalageDansVisite;

            this.visiteDephasageValeur.Value = this.parametres.Dephasage;

            this.InitListeVisites();
        }

        // ====================================================================================================
        // EVENEMENTS
        // ====================================================================================================

        // ----------------------------------------------------------------------------------------------------
        // PARAMETRES CALCUL : ACTIVATION

        /// <summary>
        /// Activation/désactivation du pas de temps.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void checkBoxPasDeTemps_CheckedChanged(object sender, EventArgs e)
        {
            this.parametres.IsPasDeTempsActif = this.parametresCheckBoxPasDeTemps.Checked;
        }

        /// <summary>
        /// Activation/désactivation du filtre haute fréquence.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void checkBoxFiltreHauteFrequence_CheckedChanged(object sender, EventArgs e)
        {
            this.parametres.IsFiltreHauteFrequenceActif = this.parametresCheckBoxFiltreHauteFrequence.Checked;
        }

        /// <summary>
        /// Activation/désactivation du filtre basse fréquence.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void checkBoxFiltreBasseFrequence_CheckedChanged(object sender, EventArgs e)
        {
            this.parametres.IsFiltreBasseFrequenceActif = parametresCheckBoxFiltreBasseFrequence.Checked;

            if (this.parametres.IsFiltreBasseFrequenceActif)
            {
                this.parametres.IsPasDeTempsActif = true;
                this.parametresCheckBoxPasDeTemps.Checked = true;
                this.parametresTextBoxPasDeTemps.Text = this.parametres.PasDeTemps.ToString();
            }
        }

        /// <summary>
        /// Activation/désactivation de la normalisation.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void checkBoxNormalisation_CheckedChanged(object sender, EventArgs e)
        {
            this.parametres.IsNormalisationActif = this.parametresCheckBoxNormalisation.Checked;
        }

        /// <summary>
        /// Activation/désactivation du seuil.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void checkBoxSeuil_CheckedChanged(object sender, EventArgs e)
        {
            this.parametres.IsSeuilActif = this.parametresCheckBoxSeuil.Checked;

            if (this.parametres.IsSeuilActif)
            {
                this.parametres.IsPasDeTempsActif = true;
                this.parametresCheckBoxPasDeTemps.Checked = true;
                this.parametresTextBoxPasDeTemps.Text = this.parametres.PasDeTemps.ToString();

                this.parametres.IsFiltreHauteFrequenceActif = true;
                this.parametresCheckBoxFiltreHauteFrequence.Checked = true;
                this.parametresTextBoxFiltreHauteFrequence.Text = this.parametres.FiltreHauteFrequence.ToString();

                this.parametres.IsFiltreBasseFrequenceActif = true;
                this.parametresCheckBoxFiltreBasseFrequence.Checked = true;
                this.parametresTextBoxFiltreBasseFrequence.Text = this.parametres.FiltreBasseFrequence.ToString();

                this.parametres.IsNormalisationActif = true;
                this.parametresCheckBoxNormalisation.Checked = true;
            }

            this.parametresBoutonCalculerIndices.Enabled = this.parametres.IsSeuilActif;
        }

        // ----------------------------------------------------------------------------------------------------
        // PARAMETRES CALCUL : MODIFICATION

        /// <summary>
        /// Modification du pas de temps.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void textBoxPasDeTemps_TextChanged(object sender, EventArgs e)
        {
            int pasDeTemps;
            int.TryParse(parametresTextBoxPasDeTemps.Text, out pasDeTemps);
            this.parametres.PasDeTemps = pasDeTemps;
        }

        /// <summary>
        /// Modification du filtre haute fréquence.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void textBoxFiltreHauteFrequence_TextChanged(object sender, EventArgs e)
        {
            double filtreHauteFrequence;
            double.TryParse(parametresTextBoxFiltreHauteFrequence.Text, out filtreHauteFrequence);
            this.parametres.FiltreHauteFrequence = filtreHauteFrequence;
        }

        /// <summary>
        /// Modification du filtre basse fréquence.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void textBoxFiltreBasseFrequence_TextChanged(object sender, EventArgs e)
        {
            int filtreBasseFrequence;
            int.TryParse(parametresTextBoxFiltreBasseFrequence.Text, out filtreBasseFrequence);
            this.parametres.FiltreBasseFrequence = filtreBasseFrequence;
        }

        /// <summary>
        /// Modification du nombre de décimales du seuil.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void textBoxSeuilNbDecimales_TextChanged(object sender, EventArgs e)
        {
            int seuilNbDecimales;
            int.TryParse(parametresTextBoxSeuilNbDecimales.Text, out seuilNbDecimales);
            this.parametres.SeuilNbDecimales = seuilNbDecimales;
        }

        /// <summary>
        /// Modification du min du seuil.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void textBoxSeuilMin_TextChanged(object sender, EventArgs e)
        {
            double seuilMin;
            double.TryParse(parametresTextBoxSeuilMin.Text, out seuilMin);
            this.parametres.SeuilMin = seuilMin;
        }

        /// <summary>
        /// Modification du max du seuil.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void textBoxSeuilMax_TextChanged(object sender, EventArgs e)
        {
            double seuilMax;
            double.TryParse(parametresTextBoxSeuilMax.Text, out seuilMax);
            this.parametres.SeuilMax = seuilMax;
        }

        /// <summary>
        /// Modification de la fenêtre de calcul des max locaux.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void textBoxSeuilFenetreCalculMaxLocaux_TextChanged(object sender, EventArgs e)
        {
            int fenetreCalculMaxLocaux;
            int.TryParse(parametresTextBoxSeuilFenetreCalculMaxLocaux.Text, out fenetreCalculMaxLocaux);
            this.parametres.FenetreCalculMaxLocaux = fenetreCalculMaxLocaux;
        }

        /// <summary>
        /// Clic sur le bouton calcul des indices.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void boutonCalculerIndices_Click(object sender, EventArgs e)
        {
            System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.WaitCursor;
            MessageBox.Show("Cette action va durer plusieurs minutes");

            var thread = Task.Run(() => CalculerIndices());
            thread.Wait();

            System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.Default;
            MessageBox.Show("Opération terminé");

            visitesOLV.BuildList(true);
        }

        // ----------------------------------------------------------------------------------------------------
        // VISITES

        /// <summary>
        /// Sélection d'une visite.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void olvVisites_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.SelectionVisite(true);
        }

        /// <summary>
        /// Clic sur le bouton Appliquer.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void boutonAppliquer_Click(object sender, EventArgs e)
        {
            this.SelectionVisite(true);
        }

        // ----------------------------------------------------------------------------------------------------
        // PARAMETRES COURBES

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        // Intervalle de temps

        /// <summary>
        /// Modification de l'intervalle de temps.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void intervalleDeTempsValeur_ValueChanged(object sender, EventArgs e)
        {
            if (visiteIntervalleValeur.Text == null || visiteIntervalleValeur.Value == 0)
            {
                this.parametres.IntervalleDeTemps = 5;
                visiteIntervalleValeur.Value = this.parametres.IntervalleDeTemps;
            }
            else
            {
                this.parametres.IntervalleDeTemps = Convert.ToInt32(visiteIntervalleValeur.Value);

            }

            this.SelectionVisite();
        }

        /// <summary>
        /// Clic sur le bouton "-" de l'intervalle de temps.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void intervalleDeTempsBoutonMoins_Click(object sender, EventArgs e)
        {
            if (visiteIntervalleValeur.Value - 1 >= visiteIntervalleValeur.Minimum)
            {
                this.parametres.IntervalleDeTemps--;
                visiteIntervalleValeur.Value = this.parametres.IntervalleDeTemps;
                this.SelectionVisite();
            }
        }

        /// <summary>
        /// Clic sur le bouton "+" de l'intervalle de temps.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void intervalleDeTempsBoutonPlus_Click(object sender, EventArgs e)
        {
            if (visiteIntervalleValeur.Value + 1 <= visiteIntervalleValeur.Maximum)
            {
                this.parametres.IntervalleDeTemps++;
                visiteIntervalleValeur.Value = this.parametres.IntervalleDeTemps;
                this.SelectionVisite();
            }
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        // Décalage dans la visite

        /// <summary>
        /// Modification du décalage dans la visite.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void decalageDansVisiteValeur_ValueChanged(object sender, EventArgs e)
        {
            if (visiteDecalageValeur.Value >= 0)
            {
                this.parametres.DecalageDansVisite = Convert.ToInt32(visiteDecalageValeur.Value);
            }
            else
            {
                this.parametres.DecalageDansVisite = 0;
                visiteDecalageValeur.Value = 0;
            }

            this.SelectionVisite();
        }

        /// <summary>
        /// Clic sur le bouton "-" du décalage dans la visite.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void decalageDansVisiteBoutonMoins_Click(object sender, EventArgs e)
        {
            if (visiteDecalageValeur.Value - 1 >= visiteDecalageValeur.Minimum)
            {
                this.parametres.DecalageDansVisite--;
                visiteDecalageValeur.Value = this.parametres.DecalageDansVisite;

                this.SelectionVisite();
            }
        }

        /// <summary>
        /// Clic sur le bouton "+" du décalage dans la visite.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void decalageDansVisiteBoutonPlus_Click(object sender, EventArgs e)
        {
            if (visiteDecalageValeur.Value + 1 <= visiteDecalageValeur.Maximum)
            {
                this.parametres.DecalageDansVisite++;
                visiteDecalageValeur.Value = this.parametres.DecalageDansVisite;

                this.SelectionVisite();
            }
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        // Déphasage

        /// <summary>
        /// Modification du déphasage.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void dephasageValeur_ValueChanged(object sender, EventArgs e)
        {
            this.parametres.Dephasage = (int)this.visiteDephasageValeur.Value;
            this.SelectionVisite(true);
        }

        /// <summary>
        /// Clic sur le bouton "-" du déphasage.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void dephasageBoutonMoins_Click(object sender, EventArgs e)
        {
            this.visiteDephasageValeur.Value--;
        }

        /// <summary>
        /// Clic sur le bouton "+" du déphasage.
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void dephasageBoutonPlus_Click(object sender, EventArgs e)
        {
            this.visiteDephasageValeur.Value++;
        }

        // ----------------------------------------------------------------------------------------------------
        // EXPORT

        /// <summary>
        /// Clic sur le bouton "Exporter les données".
        /// </summary>
        /// <param name="sender">l'objet appelant</param>
        /// <param name="e">l'évènement</param>
        private void boutonExporter_Click(object sender, EventArgs e)
        {
            SaveFileDialog SavePathExport = new SaveFileDialog();

            SavePathExport.DefaultExt = "xlsx";
            SavePathExport.InitialDirectory = Application.ExecutablePath;
            SavePathExport.FileName = "DonnéesExport";
            SavePathExport.Title = "Enregister fichier excel";

            if (SavePathExport.ShowDialog() == DialogResult.OK)
            {
                ExportExcel export = new ExportExcel();

                export.Export(
                    SavePathExport.FileName,
                    this.parametres.PasDeTemps,
                    this.parametres.FiltreHauteFrequence,
                    this.parametres.FiltreBasseFrequence,
                    this.parametres.IsNormalisationActif,
                    this.parametres.SeuilNbDecimales,
                    this.parametres.Dephasage,
                    visites);

                MessageBox.Show("La fichier à été enregistré à l'emplacement " + SavePathExport.FileName);
            }
            else
            {
                MessageBox.Show("L'enregistrement a été annulé");
            }
        }

        // ----------------------------------------------------------------------------------------------------
        // TOOL TIPS

        private void PanelCalcul_MouseHover(object sender, EventArgs e)
        {
            tooltipCalcul.Show("Nécéssite d'avoir activé le seuil pour fonctionner", this.parametresPanelBoutons);
        }

        // ====================================================================================================
        // METHODES PRIVEES
        // ====================================================================================================

        /// <summary>
        /// Initialisation de la liste des visites.
        /// </summary>
        private void InitListeVisites()
        {
            // --------------------------------------------------------------------------------
            // Colonnes

            OLVColumn colonneAuge = new OLVColumn();
            colonneAuge.AspectName = "Auge";
            colonneAuge.Text = "Auge";
            colonneAuge.Width = 40;

            OLVColumn colonneAnimal = new OLVColumn();
            colonneAnimal.AspectName = "Animal";
            colonneAnimal.Text = "Animal";
            colonneAnimal.Width = 45;

            OLVColumn colonneDate = new OLVColumn();
            colonneDate.AspectName = "DateLabel";
            colonneDate.Text = "Date";
            colonneDate.Width = 70;

            OLVColumn colonneHeureDebut = new OLVColumn();
            colonneHeureDebut.AspectName = "HeureDebutLabel";
            colonneHeureDebut.Text = "Début";
            colonneHeureDebut.Width = 60;

            OLVColumn colonneHeureFin = new OLVColumn();
            colonneHeureFin.AspectName = "HeureFinLabel";
            colonneHeureFin.Text = "Fin";
            colonneHeureFin.Width = 60;

            OLVColumn colonneDuree = new OLVColumn();
            colonneDuree.AspectName = "DureeVisiteLabel";
            colonneDuree.Text = "Durée";
            colonneDuree.Width = 60;

            OLVColumn colonneSeuil = new OLVColumn();
            colonneSeuil.AspectName = "Seuil";
            colonneSeuil.Text = "Seuil";
            colonneSeuil.Width = 60;

            OLVColumn colonneNbMaxLocaux = new OLVColumn();
            colonneNbMaxLocaux.AspectName = "NbMaxLocaux";
            colonneNbMaxLocaux.Text = "Nb max locaux";
            colonneNbMaxLocaux.Width = 60;

            OLVColumn colonneIndiceJaccard = new OLVColumn();
            colonneIndiceJaccard.AspectName = "IndiceJaccardLabel";
            colonneIndiceJaccard.Text = "Jaccard";
            colonneIndiceJaccard.Width = 60;

            OLVColumn colonneFrequenceVideo = new OLVColumn();
            colonneFrequenceVideo.AspectName = "FrequenceVideoLabel";
            colonneFrequenceVideo.Text = "Fréqu. vidéo";
            colonneFrequenceVideo.Width = 60;

            OLVColumn colonneFrequenceSeuil = new OLVColumn();
            colonneFrequenceSeuil.AspectName = "FrequenceSeuilLabel";
            colonneFrequenceSeuil.Text = "Fréqu. seuil";
            colonneFrequenceSeuil.Width = 60;

            OLVColumn colonnePourcentageVideo = new OLVColumn();
            colonnePourcentageVideo.AspectName = "PourcentageVideoLabel";
            colonnePourcentageVideo.Text = "% vidéo";
            colonnePourcentageVideo.Width = 60;

            OLVColumn colonnePourcentageSeuil = new OLVColumn();
            colonnePourcentageSeuil.AspectName = "PourcentageSeuilLabel";
            colonnePourcentageSeuil.Text = "% seuil";
            colonnePourcentageSeuil.Width = 60;

            // --------------------------------------------------------------------------------
            // Objets

            visites = VisitesService.GetVisites();

            // --------------------------------------------------------------------------------
            // Construction de la liste

            this.visitesOLV.Init(
                visites,
                colonneAuge,
                colonneAnimal,
                colonneDate,
                colonneHeureDebut,
                colonneHeureFin,
                colonneDuree,
                colonneSeuil,
                colonneNbMaxLocaux,
                colonneIndiceJaccard,
                colonneFrequenceSeuil,
                colonneFrequenceVideo,
                colonnePourcentageSeuil,
                colonnePourcentageVideo);
        }

        /// <summary>
        /// Sélection d'une visite.
        /// </summary>
        private void SelectionVisite(bool calculerFiltresEtIndices = false)
        {
            System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.WaitCursor;

            Visite visite = this.visitesOLV.SelectedObject as Visite;

            if (visite != null)
            {
                // ----------------------------------------------------------------------------------------------------
                // Affichage des valeurs de paramètres de calcul

                bool isPasDeTempsActif = this.parametresCheckBoxPasDeTemps.Checked;
                this.parametresCochePasDeTemps.Visible = isPasDeTempsActif;
                this.parametresLabelValeurPasDeTemps.Visible = isPasDeTempsActif;
                this.parametresLabelValeurPasDeTemps.Text = isPasDeTempsActif ? this.parametres.PasDeTemps.ToString() : "";

                bool isFiltreHauteFrequenceActif = this.parametresCheckBoxFiltreHauteFrequence.Checked;
                this.parametresCocheFiltreHauteFrequence.Visible = isFiltreHauteFrequenceActif;
                this.parametresLabelValeurFiltreHaute.Visible = isFiltreHauteFrequenceActif;
                this.parametresLabelValeurFiltreHaute.Text = isFiltreHauteFrequenceActif ? this.parametres.FiltreHauteFrequence.ToString() : "";

                bool isFiltreBasseFrequenceActif = this.parametresCheckBoxFiltreBasseFrequence.Checked;
                this.parametresCocheFiltreBasseFrequence.Visible = isFiltreBasseFrequenceActif;
                this.parametresLabelValeurFiltreBasseFrequence.Visible = isFiltreBasseFrequenceActif;
                this.parametresLabelValeurFiltreBasseFrequence.Text = isFiltreBasseFrequenceActif ? this.parametres.FiltreBasseFrequence.ToString() : "";

                this.parametresCocheNormalisation.Visible = this.parametresCheckBoxNormalisation.Checked;

                bool isSeuilActif = this.parametresCheckBoxSeuil.Checked;
                this.parametresCocheSeuil.Visible = isSeuilActif;
                this.parametresLabelValeurSeuilNbDecimales.Visible = isSeuilActif;
                this.parametresLabelValeurSeuilMin.Visible = isSeuilActif;
                this.parametresLabelValeurSeuilMax.Visible = isSeuilActif;
                this.parametresLabelValeurSeuilFenetreCalculMaxLocaux.Visible = isSeuilActif;
                this.parametresLabelValeurSeuilNbDecimales.Text = isSeuilActif ? this.parametres.SeuilNbDecimales.ToString() : "";
                this.parametresLabelValeurSeuilMin.Text = isSeuilActif ? this.parametres.SeuilMin.ToString() : "";
                this.parametresLabelValeurSeuilMax.Text = isSeuilActif ? this.parametres.SeuilMax.ToString() : "";
                this.parametresLabelValeurSeuilFenetreCalculMaxLocaux.Text = isSeuilActif ? this.parametres.FenetreCalculMaxLocaux.ToString() : "";

                if (calculerFiltresEtIndices)
                {
                    // ----------------------------------------------------------------------------------------------------
                    // Récupération des données

                    VisitesService.RecupererDonnees(visite, this.parametres);

                    this.visiteSeuilValeur.Text = visite.Seuil.ToString();

                    // ----------------------------------------------------------------------------------------------------
                    // Calcul des indices

                    this.CalculerIndices(visite);
                    this.visitesOLV.UpdateObject(visite);
                }

                // ----------------------------------------------------------------------------------------------------
                // Affichage des graphes

                //DateTime heureDebutOffset = visite.DateHeureDebut.AddSeconds(-this.parametres.Dephasage);
                //DateTime heureFinOffset = visite.DateHeureFin.AddSeconds(-this.parametres.Dephasage);

                //if (heureFinOffset.Subtract(heureDebutOffset.TimeOfDay).TimeOfDay >= TimeSpan.FromMinutes(this.parametres.IntervalleDeTemps))
                //{
                //    heureDebutOffset = heureDebutOffset.AddMinutes(this.parametres.DecalageDansVisite);
                //    heureFinOffset = heureDebutOffset.AddMinutes(this.parametres.IntervalleDeTemps);
                //}

                //List<RapidObject> rapidsSansFiltres = visite.GetRapidsSansFiltresPeriode(heureDebutOffset, heureFinOffset);
                //List<RapidObject> rapidsAvecFiltres = visite.GetRapidsAvecFiltresPeriode(heureDebutOffset, heureFinOffset);

                DateTime minDate = visite.DateHeureDebut.AddMinutes(this.parametres.DecalageDansVisite);
                DateTime maxDate = visite.DateHeureDebut.AddMinutes(this.parametres.DecalageDansVisite + this.parametres.IntervalleDeTemps);

                Dictionary<DateTime, int> donneesVideo = visite.GetDonneesVideoPeriode(minDate, maxDate);
                List<RapidObject> rapidsSansFiltres = visite.GetRapidsSansFiltresPeriode(minDate, maxDate);
                List<RapidObject> rapidsAvecFiltres = visite.GetRapidsAvecFiltresPeriode(minDate, maxDate);
                Dictionary<DateTime, int> donneesSeuil = visite.GetDonneesSeuilPeriode(minDate, maxDate);
                List<RapidObject> maxLocaux = visite.GetMaxLocauxPeriode(minDate, maxDate);

                this.DessinerGraphique(
                    donneesVideo,
                    rapidsSansFiltres,
                    rapidsAvecFiltres,
                    donneesSeuil,
                    maxLocaux,
                    minDate,
                    maxDate,
                    visite.MinPoidsRapidSansFiltres,
                    visite.MaxPoidsRapidSansFiltres,
                    visite.MinPoidsRapidAvecFiltres,
                    visite.MaxPoidsRapidAvecFiltres);
            }

            System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.Default;
        }

        /// <summary>
        /// Calculer les indices d'une visite.
        /// </summary>
        private void CalculerIndices(Visite visite)
        {
            Dictionary<DateTime, bool> donneesVideo = visite.DonneesVideo.ToDictionary(d => d.Key, d => d.Value != 0);
            Dictionary<DateTime, bool> donneesSeuil = visite.DonneesSeuil.ToDictionary(d => d.Key, d => d.Value != 0);

            List<bool> valeursVideo = donneesVideo.Values.ToList();
            List<bool> valeursSeuil = donneesSeuil.Values.ToList();

            visite.IndiceJaccard = Math.Round(CalculIndices.IndiceJaccard(donneesVideo, donneesSeuil), 2);

            visite.FrequenceVideo = Math.Round(CalculIndices.FrequenceTrueMinutes(valeursVideo, this.parametres.PasDeTemps), 2);
            visite.FrequenceSeuil = Math.Round(CalculIndices.FrequenceTrueMinutes(valeursSeuil, this.parametres.PasDeTemps), 2);

            visite.PourcentageVideo = Math.Round(CalculIndices.PourcentageTrue(valeursVideo), 2);
            visite.PourcentageSeuil = Math.Round(CalculIndices.PourcentageTrue(valeursSeuil), 2);
        }

        /// <summary>
        /// Calculer les indices des visites.
        /// </summary>
        private void CalculerIndices()
        {
            foreach (Visite visite in visites)
            {
                VisitesService.RecupererDonnees(visite, this.parametres);
                this.CalculerIndices(visite);
            }
        }
    }
}
