﻿using SmartCowBioControl.src.calcul;
using SmartCowBioControl.src.objects;
using SmartCowBioControl.src.objects.objects;
using System;
using System.Collections.Generic;
using System.Data.SQLite;
using System.Linq;

namespace SmartCowBioControl.src.services
{
    /// <summary>
    /// Classe de service pour les visites.
    /// </summary>
    public static class VisitesService
    {
        // ====================================================================================================
        // ATTRIBUTS
        // ====================================================================================================

        private static Dictionary<Visite, Dictionary<int, List<RapidObject>>> rapidReechantillonnes = new Dictionary<Visite, Dictionary<int, List<RapidObject>>>();

        // ====================================================================================================
        // METHODES PUBLIQUES
        // ====================================================================================================

        /// <summary>
        /// Retourne les visites.
        /// </summary>
        /// <returns>les visites</returns>
        public static List<Visite> GetVisites()
        {
            List<Visite> visites = new List<Visite>();

            SQLiteConnection connection = new SQLiteConnection("Data Source=referencedb.db");
            connection.Open();

            SQLiteCommand command = new SQLiteCommand(connection);
            command.CommandText = $"SELECT * FROM visite";

            SQLiteDataReader dr = command.ExecuteReader();

            int id = 0;
            while (dr.Read())
            {
                Visite visite = new Visite(
                    Int32.Parse(dr["id"].ToString()),
                    Int32.Parse(dr["trough"].ToString()),
                    dr["subject"].ToString(),
                    dr.GetDateTime(3),
                    dr.GetDateTime(4));

                // enlever les visites minuscules sans eating (cause crash)
                if (visite.DateHeureFin > dr.GetDateTime(3).AddSeconds(20))
                {
                    visites.Add(visite);
                }

                id = Int32.Parse(dr["id"].ToString());
            }

            connection.Close();

            return visites;
        }

        /// <summary>
        /// Récupérer les données d'une visite.
        /// </summary>
        /// <param name="visite">une visite</param>
        /// <param name="parametres">des paramètres</param>
        public static void RecupererDonnees(Visite visite, Parametres parametres)
        {
            RecupererDonneesVideo(visite);
            RecupererDonneesRapid(visite);

            visite.RapidsAvecFiltres = GetCopie(visite.RapidsSansFiltres);

            if (parametres.IsPasDeTempsActif)
            {
                visite.RapidsAvecFiltres = GetCopie(GetRapidReechantillonnes(visite, parametres.PasDeTemps));
            }

            visite.RapidsSansFiltres.ForEach(r => r.DateHeureDephasage = r.DateHeureReference.AddSeconds(parametres.Dephasage));
            visite.RapidsAvecFiltres.ForEach(r => r.DateHeureDephasage = r.DateHeureReference.AddSeconds(parametres.Dephasage));

            if (parametres.IsFiltreHauteFrequenceActif)
            {
                CalculFiltres.FiltreHauteFrequence(visite.RapidsAvecFiltres, parametres.FiltreHauteFrequence, parametres.PasDeTemps);
            }

            if (parametres.IsFiltreBasseFrequenceActif)
            {
                CalculFiltres.FiltreBasseFrequence(visite.RapidsAvecFiltres, parametres.FiltreBasseFrequence, parametres.PasDeTemps);
            }

            if (parametres.IsNormalisationActif)
            {
                CalculFiltres.Normalisation(visite.RapidsAvecFiltres);
            }

            if (parametres.IsSeuilActif)
            {
                List<RapidObject> maxLocaux;
                double seuil;
                visite.DonneesSeuil = CalculFiltres.Seuil(
                    visite.RapidsAvecFiltres,
                    parametres.FenetreCalculMaxLocaux,
                    parametres.SeuilNbDecimales,
                    parametres.SeuilMin,
                    parametres.SeuilMax,
                    out maxLocaux,
                    out seuil);
                visite.MaxLocaux = maxLocaux;
                visite.Seuil = seuil;
            }
        }

        // ====================================================================================================
        // METHODES PRIVEES
        // ====================================================================================================

        /// <summary>
        /// Récupérer les données vidéo d'une visite.
        /// </summary>
        /// <param name="visite">une visite</param>
        private static void RecupererDonneesVideo(Visite visite)
        {
            if (visite.Eatings.Count == 0)
            {
                // Eatings

                SQLiteConnection connection = new SQLiteConnection("Data Source=referencedb.db");
                connection.Open();

                SQLiteCommand command = new SQLiteCommand(connection);
                command.CommandText = $"SELECT * FROM eating WHERE id_visit == '{visite.Id}'";

                SQLiteDataReader dr = command.ExecuteReader();

                while (dr.Read())
                {
                    visite.Eatings.Add(new Eating(Int32.Parse(dr["id_visit"].ToString()), dr.GetDateTime(2), dr.GetDateTime(3)));

                }

                connection.Close();

                // Données vidéo

                if (visite.Eatings.Count > 0)
                {
                    Eating eatingCourant = visite.Eatings[0];

                    if (eatingCourant == null)
                    {
                        return;
                    }

                    int compteur = 0;

                    for (DateTime dateHeure = visite.DateHeureDebut; dateHeure <= visite.DateHeureFin; dateHeure = dateHeure.AddSeconds(1))
                    {
                        if (dateHeure < eatingCourant.DateHeureDebut)
                        {
                            visite.DonneesVideo.Add(dateHeure, 0);
                        }
                        else if (dateHeure < eatingCourant.DateHeureFin)
                        {
                            visite.DonneesVideo.Add(dateHeure, 1);
                        }
                        else
                        {
                            compteur++;
                            if (compteur >= visite.Eatings.Count)
                            {
                                compteur = 0;
                            }
                            eatingCourant = visite.Eatings[compteur];

                            visite.DonneesVideo.Add(dateHeure, 0);
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Récupère les données Rapid d'une visite.
        /// </summary>
        /// <param name="visite">une visite</param>
        private static void RecupererDonneesRapid(Visite visite)
        {
            if (visite.RapidsSansFiltres.Count == 0)
            {
                SQLiteConnection connection = new SQLiteConnection("Data Source=rapiddb.db");
                connection.Open();

                SQLiteCommand command = new SQLiteCommand(connection);
                command.CommandText = $"SELECT * FROM 'Auge {visite.Auge}' WHERE date BETWEEN '{visite.DateHeureDebut.ToString("yyyy-MM-dd HH:mm:ss.f")}' AND '{visite.DateHeureFin.ToString("yyyy-MM-dd HH:mm:ss.f")}'";

                SQLiteDataReader dr = command.ExecuteReader();

                while (dr.Read())
                {
                    int animal = Convert.ToInt32(dr["animal"]);
                    DateTime dateHeure = dr.GetDateTime(2);
                    int poids = Convert.ToInt32(dr["poid"]);

                    visite.RapidsSansFiltres.Add(new RapidObject(animal, dateHeure, poids));
                }

                dr.Close();
            }
        }

        private static List<RapidObject> GetRapidReechantillonnes(Visite visite, int pasDeTemps)
        {
            if (!rapidReechantillonnes.ContainsKey(visite))
            {
                rapidReechantillonnes.Add(visite, new Dictionary<int, List<RapidObject>>());
            }

            if (!rapidReechantillonnes[visite].ContainsKey(pasDeTemps))
            {
                rapidReechantillonnes[visite].Add(pasDeTemps, Reechantillonnage.GetRapidReechantillonnes(visite.RapidsSansFiltres, pasDeTemps, visite.DateHeureDebut, visite.DateHeureFin));
            }

            return rapidReechantillonnes[visite][pasDeTemps];
        }

        private static List<RapidObject> GetCopie(List<RapidObject> rapids)
        {
            return rapids.Select(r => r.Clone).ToList();
        }
    }
}
