﻿using BrightIdeasSoftware;
using System;
using System.Collections;

namespace SmartCowBioControl.src.util
{
    /// <summary>
    /// Extension de ObjectListView.
    /// </summary>
    public class MyObjectListView : ObjectListView
    {
        // ====================================================================================================
        // PROPRIETES
        // ====================================================================================================

        /// <summary>
        /// Le type des objets.
        /// </summary>
        public Type TypeObjets { get; private set; }

        // ====================================================================================================
        // CONSTRUCTEURS
        // ====================================================================================================

        /// <summary>
        /// Constructeur.
        /// </summary>
        public MyObjectListView()
        {
            // Initialisations des propriétés
            this.AllowColumnReorder = true;
            this.AllowDrop = true;
            this.EmptyListMsg = "Aucun élément";
            this.FullRowSelect = true;
            this.HeaderUsesThemes = false;
            this.HeaderWordWrap = true;
            this.HideSelection = false;
            this.OwnerDraw = true;
            this.ShowCommandMenuOnRightClick = true;
            this.ShowGroups = false;
            this.ShowItemToolTips = true;
            this.SortGroupItemsByPrimaryColumn = false;
            this.TintSortColumn = true;
            this.UnfocusedSelectedBackColor = SelectedBackColorOrDefault;
            this.UnfocusedSelectedForeColor = SelectedForeColorOrDefault;
            this.UseAlternatingBackColors = true;
            this.UseFilterIndicator = true;
            this.UseFiltering = true;
            this.UseHotItem = true;
            this.View = System.Windows.Forms.View.Details;

            // Françisation des menus
            this.MenuLabelSortAscending = "Tri ascendant";
            this.MenuLabelSortDescending = "Tri descendant";
            this.MenuLabelGroupBy = "Grouper par '{0}'";
            this.MenuLabelLockGroupingOn = "Bloquer le regroupement sur '{0}'";
            this.MenuLabelUnlockGroupingOn = "Débloquer le regroupement sur '{0}'";
            this.MenuLabelTurnOffGroups = "Dégrouper";
            this.MenuLabelUnsort = "Annuler tri";
            FilterMenuBuilder.CLEAR_ALL_FILTERS_LABEL = "Effacer les filtres";
            FilterMenuBuilder.APPLY_LABEL = "Appliquer";
            FilterMenuBuilder.FILTERING_LABEL = "Filtrer";
            FilterMenuBuilder.SELECT_ALL_LABEL = "Sélectionner Tout";
        }

        // ====================================================================================================
        // METHODES PUBLIQUES
        // ====================================================================================================

        /// <summary>
        /// Initialisation.
        /// </summary>
        /// <param name="colonnes">la liste des colonnes</param>
        public void Init(params OLVColumn[] colonnes)
        {
            this.Init(null, colonnes);
        }

        /// <summary>
        /// Initialisation.
        /// </summary>
        /// <param name="objects">la collection d'objets</param>
        /// <param name="colonnes">la liste des colonnes</param>
        public void Init(IEnumerable objects, params OLVColumn[] colonnes)
        {
            // Colonnes
            this.Clear();
            this.AllColumns.Clear();
            this.AllColumns.AddRange(colonnes);
            this.RebuildColumns();

            // Objets
            if (objects != null)
            {
                this.SetObjects(objects);
            }
        }

        // ====================================================================================================
        // METHODES PRIVEES
        // ====================================================================================================

        /// <summary>
        /// Initialisation des composants.
        /// </summary>
        private void InitializeComponent()
        {
            ((System.ComponentModel.ISupportInitialize)(this)).BeginInit();
            this.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this)).EndInit();
            this.ResumeLayout(false);
        }
    }
}
