using Microsoft.VisualStudio.TestTools.UnitTesting;
using SmartCowBioControl.src.form;
using SmartCowBioControl.src.objects;
using System.Collections.Generic;
using System;

namespace TestFonctions
{
    /// <summary>
    /// Classe de test des filtres et indices
    /// </summary>
    [TestClass]
    public class TestFiltres
    {
        [TestMethod]
        public void TestFiltreDecalage()
        {
            //le param de dcalage de seconde
            double DecalageTemps = 3.0;
            DephasageRapid Decalage = new DephasageRapid();

            //declarer la list contenant des valeurs
            List<RapidObject> rapids = new List<RapidObject>();
            //remplir la liste de faux objets
            for (int i = 0; i < 20; i++)
            {
                rapids.Add(new RapidObject(i, System.DateTime.Now.AddSeconds(i), i));
            }

            //dclarer la liste qui va garder les rsultats sans garder la rfrence  la list rapids
            List<RapidObject> ListAvantModif = new List<RapidObject>();

            //remplire la nouvelle liste sans la reference  la liste rapids
            foreach (var item in rapids)
            {
                ListAvantModif.Add(new RapidObject(item.m_weight, item.m_date, item.m_animal));
            }

            List<RapidObject> ListApresModif = Decalage.Dephasage(rapids, DecalageTemps);

            //verifier la taille des deux list
            if (ListApresModif.Count != rapids.Count)
            {
                Assert.Fail("Pas la mme taille entre le rsultat et la liste d'origine");
            }
            //si les items sont nuls
            CollectionAssert.AllItemsAreNotNull(ListApresModif);

            //vrifier que le dcalage est bien effectu 
            for (int i = 0; i < ListApresModif.Count; i++)
            {
                if (ListAvantModif[i].m_date.AddSeconds(DecalageTemps) != ListApresModif[i].m_date)
                {
                    Assert.Fail("Le dcalage ne correspond pas");
                }
            }
        }


        [TestMethod]
        public void TestFiltreBasseFrequence()
        {
            Filtres filtreBasseFrequence = new Filtres();
            int PasDeTemps = 1;
            int param = 4;
            //declarer la list contenant des valeurs
            List<RapidObject> rapids = new List<RapidObject>();
            //remplir la liste avec des valeurs connues
            rapids.Add(new RapidObject(4, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(6, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(2, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(7, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(8, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(1, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(9, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(4, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(5, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(3, System.DateTime.Now, 1));


            //dclarer la liste qui va garder les rsultats sans garder la rfrence  la list rapids
            List<RapidObject> ListAvantModif = new List<RapidObject>();

            //remplire la nouvelle liste sans la reference  la liste rapids
            foreach (var item in rapids)
            {
                ListAvantModif.Add(new RapidObject(item.m_weight, item.m_date, item.m_animal));
            }

            List<RapidObject> ListApresModif = filtreBasseFrequence.FiltreBasseFrequence(rapids, param, PasDeTemps);
            Console.WriteLine("\n\n\n");
            Console.WriteLine("Affichage avant modif");
            foreach (var item in ListAvantModif)
            {
                Console.WriteLine(item.m_weight);
            }
            Console.WriteLine("------------------");
            Console.WriteLine("\n\n\n");
            Console.WriteLine("Affichage aprs modif");
            foreach (var item in ListApresModif)
            {
                Console.WriteLine(item.m_weight);
            }

            List<int> ResultatAttendu = new List<int>() { 2, 4, 1, 6, 7, 0, 6, 1, 2, 0 };

            List<int> ResultatAttenduBis = new List<int>() { 3, 5, 1, 6, 7, 0, 8, 3, 4, 2 };


            for (int i = 0; i < ListApresModif.Count; i++)
            {
                if (ListApresModif[i].m_weight != ResultatAttendu[i])
                {
                    Assert.Fail("Le rsultat attendu n'est pas bon au " + i + "e rsultat");
                }
            }

        }


        [TestMethod]
        public void TestFiltreNormalisation()
        {

            Normalisation normalistion = new Normalisation();


            //declarer la list contenant des valeurs
            List<RapidObject> rapids = new List<RapidObject>();
            //remplir la liste avec des valeurs connues
            rapids.Add(new RapidObject(4, System.DateTime.Now, 9995));
            rapids.Add(new RapidObject(6, System.DateTime.Now, 9901));
            rapids.Add(new RapidObject(2, System.DateTime.Now, 9909));
            rapids.Add(new RapidObject(7, System.DateTime.Now, 532));
            rapids.Add(new RapidObject(8, System.DateTime.Now, 0054));
            rapids.Add(new RapidObject(1, System.DateTime.Now, 7852));
            rapids.Add(new RapidObject(9, System.DateTime.Now, 2584));
            rapids.Add(new RapidObject(4, System.DateTime.Now, 2674));
            rapids.Add(new RapidObject(5, System.DateTime.Now, 4222));
            rapids.Add(new RapidObject(3, System.DateTime.Now, 6221));


            //dclarer la liste qui va garder les rsultats sans garder la rfrence  la list rapids
            List<RapidObject> ListAvantModif = new List<RapidObject>();

            //remplire la nouvelle liste sans la reference  la liste rapids
            foreach (var item in rapids)
            {
                ListAvantModif.Add(new RapidObject(item.m_weight, item.m_date, item.m_animal));
            }

            List<RapidObject> ListApresModif = normalistion.NormalisationFiltre(rapids);

            List<double> ResultatAttendu = new List<double>() { (double)4 / 9, (double)6 / 9, (double)2 / 9, (double)7 / 9, (double)8 / 9, (double)1 / 9, 1.0, (double)4 / 9, (double)5 / 9, (double)3 / 9 };
            for (int i = 0; i < ListApresModif.Count; i++)
            {
                if (ListApresModif[i].m_weight != ResultatAttendu[i])
                {

                    Console.WriteLine(ResultatAttendu[i]);
                    Console.WriteLine(ListApresModif[i].m_weight);

                    Assert.Fail("Le rsultat attendu n'est pas bon au " + i + "e rsultat (Le test choue  cause du rounding pour les graphique)");
                }
            }
        }

        [TestMethod]
        public void TestFiltreHauteFrequence()
        {
            Filtres filtreHauteFrequence = new Filtres();
            int PasDeTemps = 2;
            int param = 5;
            //declarer la list contenant des valeurs
            List<RapidObject> rapids = new List<RapidObject>();
            //remplir la liste avec des valeurs connues
            rapids.Add(new RapidObject(4, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(6, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(2, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(7, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(8, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(1, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(9, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(4, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(5, System.DateTime.Now, 1));
            rapids.Add(new RapidObject(3, System.DateTime.Now, 1));


            //dclarer la liste qui va garder les rsultats sans garder la rfrence  la list rapids
            List<RapidObject> ListFiltrebassefrequence = new List<RapidObject>();

            //remplire la nouvelle liste sans la reference  la liste rapids
            foreach (var item in rapids)
            {
                ListFiltrebassefrequence.Add(new RapidObject(item.m_weight, item.m_date, item.m_animal));
            }

            //test quand le filtre est  2 
            List<RapidObject> ListFiltrebassefrequencePasDeTempsModif = filtreHauteFrequence.FiltreHauteFrequence(rapids, param, PasDeTemps);

            List<double> ResultatAttenduPasDeTempsModif = new List<double>() { 5, 4, 5, 5.666666666666667, 5.333333333333333, 6, 4.666666666666667, 6, 4, 4 };

            for (int i = 0; i < ListFiltrebassefrequencePasDeTempsModif.Count; i++)
            {
                if (ListFiltrebassefrequencePasDeTempsModif[i].m_weight != ResultatAttenduPasDeTempsModif[i])
                {
                    Assert.Fail("Le rsultat attendu n'est pas bon au " + i + "e rsultat");
                }
            }

            //test quand le filtre est  1 
            ListFiltrebassefrequence = filtreHauteFrequence.FiltreHauteFrequence(ListFiltrebassefrequence, param, 1);


            List<double> ResultatAttendu = new List<double>() { 4, 4.75, 5.4, 4.8, 5.4, 5.8, 5.4, 4.4, 5.25, 4 };
            for (int i = 0; i < ListFiltrebassefrequence.Count; i++)
            {
                if (ListFiltrebassefrequence[i].m_weight != ResultatAttendu[i])
                {
                    Assert.Fail("Le rsultat attendu n'est pas bon au " + i + "e rsultat");
                }
            }
        }

        [TestMethod]
        public void TestIndiceJaccard()
        {

            Filtres filtre = new Filtres();
            Dictionary<DateTime, int> Video = new Dictionary<DateTime, int>();
            Dictionary<DateTime, double> seuil = new Dictionary<DateTime, double>();
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 0);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);



            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);

            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);

            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);

            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, 0);

            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);

            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);

            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, 0);

            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);

            if (filtre.IndiceJaccard(Video, seuil) != 0.45)
            {

                Assert.Fail("Le rsultat de l'indice ne correspond pas");

            }
        }


        [TestMethod]
        public void TestPourcentageDuree()
        {
            Console.WriteLine("a passe la ?");
            Filtres filtre = new Filtres();
            Dictionary<DateTime, int> Video = new Dictionary<DateTime, int>();
            Dictionary<DateTime, double> seuil = new Dictionary<DateTime, double>();
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 0);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);



            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);

            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);

            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);

            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, 0);

            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);

            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);

            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, 0);

            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            if (filtre.PourcentageDuree(Video, seuil)[1]  != 0.4666666667 && filtre.PourcentageDuree(Video, seuil)[0] != 0.5)
            {

                Assert.Fail("Le rsultat  ne correspond pas");

            }
        }

        [TestMethod]
        public void TestFrequence()
        {

            Filtres filtre = new Filtres();
            Dictionary<DateTime, int> Video = new Dictionary<DateTime, int>();
            Dictionary<DateTime, double> seuil = new Dictionary<DateTime, double>();
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 0);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);
            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);

            Video.Add(DateTime.Now, 0);
            Video.Add(DateTime.Now, 1);



            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);

            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);

            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);

            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, 0);

            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);

            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, -1);

            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, -1);
            seuil.Add(DateTime.Now, 0);

            seuil.Add(DateTime.Now, 0);
            seuil.Add(DateTime.Now, 0);
            double[] result = filtre.Frequence(Video, seuil);

            if (result[0] != 18 && result[1] != 14)
            {

                Assert.Fail("Le rsultat  ne correspond pas");

            }

        }
    }


}

