﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;

namespace SmartCowBioControl.src.calcul
{
    /// <summary>
    /// Classe pour le calcul des indices.
    /// </summary>
    public static class CalculIndices
    {
        // ====================================================================================================
        // METHODES PUBLIQUES
        // ====================================================================================================

        /// <summary>
        /// Calcul du pourcentage de true.
        /// </summary>
        /// <param name="liste"></param>
        /// <returns></returns>
        public static double PourcentageTrue(List<bool> liste)
        {
            return liste == null || liste.Count == 0 ? 0 : liste.Count(l => l) / (double)liste.Count;
        }

        /// <summary>
        /// Calcul de la fréquence à true en minutes.
        /// </summary>
        /// <param name="liste"></param>
        /// <param name="pasDeTempsSecondes"></param>
        /// <returns></returns>
        public static double FrequenceTrueMinutes(List<bool> liste, int pasDeTempsSecondes, BackgroundWorker backgroundWorker = null)
        {
            if (liste == null ||
                liste.Count == 0)
            {
                return 0;
            }

            int nbPassageATrue = 0;

            bool valeurPrecedente = false;
            foreach (bool valeur in liste)
            {
                if (backgroundWorker != null && backgroundWorker.CancellationPending)
                {
                    return 0;
                }

                if (!valeurPrecedente && valeur)
                {
                    nbPassageATrue++;
                }

                valeurPrecedente = valeur;
            }

            TimeSpan duree = TimeSpan.FromSeconds(liste.Count * pasDeTempsSecondes);

            return nbPassageATrue / duree.TotalMinutes;
        }
    }
}
