﻿using ClosedXML.Excel;
using SmartCowBioControl.src.objects.objects;
using System.Collections.Generic;

namespace SmartCowBioControl.src.export
{
    /// <summary>
    /// Exports des données en Excel.
    /// </summary>
    public class ExportExcel
    {
        public void Export(
            string filePath,
            int pasDeTemps,
            double filtreHauteFreq,
            int filtreBasseFreq,
            bool normalisation,
            int seuilNbDecimales,
            IEnumerable<Visite> visites)
        {
            IXLWorkbook wb = new XLWorkbook();
            IXLWorksheet ws = wb.AddWorksheet("Filtres");

            int ligne = 1;

            ws.Cell(ligne, 1).Value = "Pas de Temps (s)";
            ws.Cell(ligne, 2).Value = "Filtre haute fréquence (s)";
            ws.Cell(ligne, 3).Value = "Filtre basse fréquence (s)";
            ws.Cell(ligne, 4).Value = "Normalisation";
            ws.Cell(ligne, 5).Value = "Seuil nb decimales";

            ws.Cell(ligne, 8).Value = "Auge";
            ws.Cell(ligne, 9).Value = "Animal";
            ws.Cell(ligne, 10).Value = "Date";
            ws.Cell(ligne, 11).Value = "Heure Debut";
            ws.Cell(ligne, 12).Value = "Heure Fin";
            ws.Cell(ligne, 13).Value = "Durée Visite";
            ws.Cell(ligne, 14).Value = "Seuil";
            ws.Cell(ligne, 15).Value = "Nb max locaux";
            ws.Cell(ligne, 16).Value = "Fréquence Seuil";
            ws.Cell(ligne, 17).Value = "Pourcentage Seuil";

            foreach (var visite in visites)
            {
                ligne++;

                ws.Cell(ligne, 1).Value = pasDeTemps;
                ws.Cell(ligne, 2).Value = filtreHauteFreq;
                ws.Cell(ligne, 3).Value = filtreBasseFreq;
                ws.Cell(ligne, 4).Value = normalisation.ToString();
                ws.Cell(ligne, 5).Value = seuilNbDecimales;

                ws.Cell(ligne, 8).Value = visite.Auge;
                ws.Cell(ligne, 9).Value = visite.Animal;
                ws.Cell(ligne, 10).Value = visite.DateLabel.ToString();
                ws.Cell(ligne, 11).Value = visite.DateHeureDebut.ToString();
                ws.Cell(ligne, 12).Value = visite.DateHeureFin.ToString();
                ws.Cell(ligne, 13).Value = visite.DureeVisite.ToString();
                ws.Cell(ligne, 14).Value = visite.Seuil;
                ws.Cell(ligne, 15).Value = visite.NbMaxLocaux;
                ws.Cell(ligne, 16).Value = visite.FrequenceSeuil;
                ws.Cell(ligne, 17).Value = visite.PourcentageSeuil;
            }

            wb.SaveAs(filePath);
        }
    }
}
