﻿using SmartCowBioControl.src.objects;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms.DataVisualization.Charting;

namespace SmartCowBioControl.src.form
{
    /// <summary>
    /// Fenêtre principale.
    /// Partie de classe pour les graphes.
    /// </summary>
    public partial class FenetrePrincipale
    {
        // ====================================================================================================
        // METHODES PRIVEES
        // ====================================================================================================

        /// <summary>
        /// Dessiner le graphique.
        /// </summary>
        /// <param name="rapidSansFiltres"></param>
        /// <param name="rapidsAvecFiltres"></param>
        /// <param name="donneesSeuil"></param>
        /// <param name="maxLocaux"></param>
        /// <param name="minDate"></param>
        /// <param name="maxDate"></param>
        /// <param name="minPoidsSansFiltres"></param>
        /// <param name="maxPoidsSansFiltres"></param>
        /// <param name="minPoidsAvecFiltres"></param>
        /// <param name="maxPoidsAvecFiltres"></param>
        private void DessinerGraphique(
            List<RapidObject> rapidSansFiltres,
            List<RapidObject> rapidsAvecFiltres,
            Dictionary<DateTime, int> donneesSeuil,
            List<RapidObject> maxLocaux,
            DateTime minDate,
            DateTime maxDate,
            double minPoidsSansFiltres,
            double maxPoidsSansFiltres,
            double minPoidsAvecFiltres,
            double maxPoidsAvecFiltres)
        {
            double minPoidsSansFiltresRound = Math.Round(minPoidsSansFiltres, 2);
            double maxPoidsSansFiltresRound = Math.Round(maxPoidsSansFiltres, 2);
            double minPoidsAvecFiltresRound = Math.Round(minPoidsAvecFiltres, 2);
            double maxPoidsAvecFiltresRound = Math.Round(maxPoidsAvecFiltres, 2);

            ChartArea chartAreaBrut = this.visiteChart.ChartAreas[0];
            ChartArea chartAreaFiltre = this.visiteChart.ChartAreas[1];

            Series serieRapidSansFiltres = this.visiteChart.Series[0];
            Series serieRapidAvecFiltres = this.visiteChart.Series[1];
            Series serieSeuil = this.visiteChart.Series[2];
            Series serieMaxLocaux = this.visiteChart.Series[3];

            // --------------------------------------------------------------------------------
            // DONNEES BRUTES

            chartAreaBrut.AxisX.Interval = 1;
            chartAreaBrut.AxisX.IntervalOffset = 1;
            chartAreaBrut.AxisX.IntervalType = DateTimeIntervalType.Minutes;
            chartAreaBrut.AxisX.LabelStyle.Angle = -45;
            chartAreaBrut.AxisX.LabelStyle.Format = "HH:mm:ss";
            chartAreaBrut.AxisX.MajorGrid.LineColor = Color.DarkGray;
            chartAreaBrut.AxisX.Maximum = maxDate.ToOADate();
            chartAreaBrut.AxisX.Minimum = minDate.ToOADate();

            chartAreaBrut.AxisY.IsStartedFromZero = false;
            chartAreaBrut.AxisY.MajorGrid.LineColor = Color.DarkGray;
            chartAreaBrut.AxisY.Maximum = maxPoidsSansFiltresRound;
            chartAreaBrut.AxisY.Minimum = minPoidsSansFiltresRound;

            // Courbe Rapid sans filtres
            serieRapidSansFiltres.Points.Clear();
            if (rapidSansFiltres != null)
            {
                foreach (RapidObject rapid in rapidSansFiltres)
                {
                    serieRapidSansFiltres.Points.AddXY(rapid.DateHeure, Math.Round(rapid.Poids, 2));
                }
            }

            // --------------------------------------------------------------------------------
            // DONNEES FILTRES

            chartAreaFiltre.AxisX.Interval = 1;
            chartAreaFiltre.AxisX.IntervalOffset = 1;
            chartAreaFiltre.AxisX.IntervalType = DateTimeIntervalType.Minutes;
            chartAreaFiltre.AxisX.LabelStyle.Angle = -45;
            chartAreaFiltre.AxisX.LabelStyle.Format = "HH:mm:ss";
            chartAreaFiltre.AxisX.MajorGrid.LineColor = Color.DarkGray;
            chartAreaFiltre.AxisX.Maximum = maxDate.ToOADate();
            chartAreaFiltre.AxisX.Minimum = minDate.ToOADate();

            chartAreaFiltre.AxisY.InterlacedColor = Color.DarkGray;
            chartAreaFiltre.AxisY.IsStartedFromZero = false;
            chartAreaFiltre.AxisY.MajorGrid.LineColor = Color.DarkGray;
            chartAreaFiltre.AxisY.Maximum = maxPoidsAvecFiltresRound;
            chartAreaFiltre.AxisY.Minimum = this.parametres.IsSeuilActif ? -1 : minPoidsAvecFiltresRound;

            // Courbe Rapid avec filtres
            serieRapidAvecFiltres.Points.Clear();
            if (rapidsAvecFiltres != null)
            {
                foreach (var rapid in rapidsAvecFiltres)
                {
                    serieRapidAvecFiltres.Points.AddXY(rapid.DateHeure, Math.Round(rapid.Poids, 2));
                }
            }

            // Courbes seuil et max locaux
            serieSeuil.Points.Clear();
            serieMaxLocaux.Points.Clear();
            if (this.parametres.IsSeuilActif && donneesSeuil != null)
            {
                foreach (var valeurSeuil in donneesSeuil)
                {
                    serieSeuil.Points.AddXY(valeurSeuil.Key, valeurSeuil.Value);
                }

                if (maxLocaux != null)
                {
                    foreach (var maxLocal in maxLocaux)
                    {
                        serieMaxLocaux.Points.AddXY(maxLocal.DateHeure, Math.Round(maxLocal.Poids, 2));
                    }
                }
            }

            // --------------------------------------------------------------------------------

            chartAreaFiltre.AlignWithChartArea = "ChartArea1";

            this.visiteChart.Invalidate();
        }
    }
}
