﻿using SmartCowBioControl.Properties;
using SmartCowBioControl.src.database;
using SmartCowBioControl.src.objects;
using SmartCowBioControl.src.objects.fichiers;
using SmartCowBioControl.src.util;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows.Forms;

namespace SmartCowBioControl.src.form
{
    public partial class SelectionDonneesForm : Form
    {
        // ====================================================================================================
        // ATTRIBUTS
        // ====================================================================================================

        private string sauvegardeDossierLogFiles;
        private string sauvegardeDossierRapid;
        private DateTime sauvegardeDateDebut;
        private DateTime sauvegardeDateFin;

        private List<BioControlVisite> visites;
        private List<BioControlPoids> poids;

        // ====================================================================================================
        // PROPRIETES
        // ====================================================================================================

        /// <summary>
        /// Le dossier LogFiles.
        /// </summary>
        private string DossierLogFiles
        {
            get => Settings.Default.DossierLogFiles;
            set
            {
                Settings.Default.DossierLogFiles = value;
                Settings.Default.Save();
                this.textBoxDossierLogFiles.Text = value;
            }
        }

        /// <summary>
        /// Le dossier Rapid.
        /// </summary>
        private string DossierRapid
        {
            get => Settings.Default.DossierRapid;
            set
            {
                Settings.Default.DossierRapid = value;
                Settings.Default.Save();
                this.textBoxDossierRapid.Text = value;
            }
        }

        /// <summary>
        /// La date de début.
        /// </summary>
        private DateTime DateDebut
        {
            get => Settings.Default.DateDebut;
            set
            {
                Settings.Default.DateDebut = value;
                Settings.Default.Save();
            }
        }

        /// <summary>
        /// La date de fin.
        /// </summary>
        private DateTime DateFin
        {
            get => Settings.Default.DateFin;
            set
            {
                Settings.Default.DateFin = value;
                Settings.Default.Save();
            }
        }

        // ====================================================================================================
        // CONSTRUCTEURS
        // ====================================================================================================

        public SelectionDonneesForm()
        {
            InitializeComponent();

            if (Utilitaire.IsDateNull(this.DateDebut))
            {
                this.DateDebut = DateTime.Today;
            }

            if (Utilitaire.IsDateNull(this.DateFin))
            {
                this.DateFin = DateTime.Today;
            }

            this.sauvegardeDossierLogFiles = this.DossierLogFiles;
            this.sauvegardeDossierRapid = this.DossierRapid;
            this.sauvegardeDateDebut = this.DateDebut;
            this.sauvegardeDateFin = this.DateFin;

            this.textBoxDossierLogFiles.Text = this.DossierLogFiles;
            this.textBoxDossierRapid.Text = this.DossierRapid;
            this.dtpDebut.Value = this.DateDebut;
            this.dtpFin.Value = this.DateFin;
        }

        // ====================================================================================================
        // EVENEMENTS
        // ====================================================================================================

        private void boutonModifierDossierLogFiles_Click(object sender, EventArgs e)
        {
            this.ModifierDossierLogFiles();
        }

        private void boutonModifierDossierRapid_Click(object sender, EventArgs e)
        {
            this.ModifierDossierRapid();
        }

        private void boutonValider_Click(object sender, EventArgs e)
        {
            this.Valider();
        }

        // ====================================================================================================
        // METHODES PRIVEES
        // ====================================================================================================

        // ----------------------------------------------------------------------------------------------------
        // DOSSIERS

        /// <summary>
        /// Modification du dossier des fichiers LogFiles.
        /// </summary>
        private void ModifierDossierLogFiles()
        {
            FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog();
            folderBrowserDialog.SelectedPath = this.DossierLogFiles;

            if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
            {
                string[] fichiersBC = Directory.GetFiles(folderBrowserDialog.SelectedPath, FichierLog.FORMAT_FICHIER);
                string[] fichiersVisites = Directory.GetFiles(folderBrowserDialog.SelectedPath, FichierVisites.FORMAT_FICHIER);

                if (fichiersBC.Length == 0 && fichiersVisites.Length == 0)
                {
                    AfficheurMessage.Instance.AfficherMessageErreur("Le dossier sélectionné ne contient pas de fichier LOG ni de fichier de visites.");
                }
                else
                {
                    this.DossierLogFiles = folderBrowserDialog.SelectedPath;
                }
            }
        }

        /// <summary>
        /// Modification du dossier des fichiers Rapid.
        /// </summary>
        private void ModifierDossierRapid()
        {
            FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog();
            folderBrowserDialog.SelectedPath = this.DossierRapid;

            if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
            {
                if (Directory.GetFiles(folderBrowserDialog.SelectedPath, "??????.txt").Length == 0)
                {
                    AfficheurMessage.Instance.AfficherMessageErreur("Le dossier sélectionné ne contient pas de fichier Rapid.");
                }
                else
                {
                    this.DossierRapid = folderBrowserDialog.SelectedPath;
                }
            }
        }

        // ----------------------------------------------------------------------------------------------------

        private void Valider()
        {
            bool isChangementDonnees =
                !string.IsNullOrWhiteSpace(this.DossierLogFiles) &&
                !string.IsNullOrWhiteSpace(this.DossierRapid) &&
                this.ValiderDates() &&
                (
                    this.checkBoxForcerRelecture.Checked ||
                    !string.Equals(this.sauvegardeDossierLogFiles, this.DossierLogFiles) ||
                    !string.Equals(this.sauvegardeDossierRapid, this.DossierRapid) ||
                    this.sauvegardeDateDebut != this.DateDebut ||
                    this.sauvegardeDateFin != this.DateFin
                );

            if (isChangementDonnees)
            {
                TraitementEnCoursForm.Afficher($"Lecture des fichiers{Environment.NewLine}et création de la base de données...", this.LireFichiers, this);
            }
        }

        private bool ValiderDates()
        {
            bool isDatesValides = this.dtpDebut.Value.Date <= this.dtpFin.Value.Date;

            if (isDatesValides)
            {
                this.DateDebut = this.dtpDebut.Value.Date;
                this.DateFin = this.dtpFin.Value.Date;
            }
            else
            {
                AfficheurMessage.Instance.AfficherMessageErreur("Les dates sont invalides.");
            }

            return isDatesValides;
        }

        /// <summary>
        /// Lecture des fichiers.
        /// </summary>
        private void LireFichiers()
        {
            bool erreur = false;

            try
            {
                DatabaseService.InitDatabase();
                DatabaseService.BeginTransaction();

                DateTime dateDebut = this.DateDebut;
                DateTime dateFin = this.DateFin.AddDays(1);

                // --------------------------------------------------------------------------------
                // Lecture des visites

                this.visites = new List<BioControlVisite>();
                List<DateTime> dates = new List<DateTime>();

                for (DateTime date = dateDebut; date <= dateFin; date = date.AddDays(1))
                {
                    this.visites.Clear();
                    this.visites.AddRange(new FichierLog(this.DossierLogFiles, date).GetVisites());
                    DatabaseService.InsertVisites(this.visites);

                    if (this.visites.Count > 0)
                    {
                        dates.Add(this.visites.Min(v => v.DateHeureDebut));
                        dates.Add(this.visites.Max(v => v.DateHeureFin));
                    }
                }

                this.visites.Clear();
                this.visites.AddRange(FichierVisites.GetVisites(this.DossierLogFiles, dateDebut, dateFin.AddSeconds(-1)));
                DatabaseService.InsertVisites(this.visites);

                if (this.visites.Count > 0)
                {
                    dates.Add(this.visites.Min(v => v.DateHeureDebut));
                    dates.Add(this.visites.Max(v => v.DateHeureFin));
                }

                DateTime dateVisiteMin = dates.Min().Date;
                DateTime dateVisiteMax = dates.Max().Date;

                // --------------------------------------------------------------------------------
                // Lecture des poids

                this.poids = new List<BioControlPoids>();

                for (DateTime date = dateVisiteMin; date <= dateVisiteMax; date = date.AddDays(1))
                {
                    this.poids.Clear();
                    this.poids.AddRange(new FichierRapid(this.DossierRapid, date).GetPoids());
                    DatabaseService.InsertRapid(this.poids);
                }
            }
            catch (Exception e)
            {
                erreur = true;
                AfficheurMessage.Instance.AfficherMessageErreur("Erreur lors de la lecture des fichiers : " + e.Message);
            }
            finally
            {
                DatabaseService.CloseTransaction(erreur);
            }
        }
    }
}
