﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SmartCowBioControl.src.form
{
    public partial class TraitementEnCoursForm : Form
    {
        // ====================================================================================================
        // PROPRIETES
        // ====================================================================================================

        /// <summary>
        /// Le worker.
        /// </summary>
        public Action Worker { get; private set; }

        // ====================================================================================================
        // CONSTRUCTEURS
        // ====================================================================================================

        /// <summary>
        /// Constructeur.
        /// </summary>
        /// <param name="worker">un worker</param>
        public TraitementEnCoursForm(Action worker)
        {
            InitializeComponent();

            if (worker == null)
            {
                throw new ArgumentNullException();
            }

            this.Worker = worker;
        }

        // ====================================================================================================
        // METHODES PUBLIQUES
        // ====================================================================================================

        /// <summary>
        /// Afficher une fenêtre de traitement.
        /// </summary>
        /// <param name="texte">un texte</param>
        /// <param name="worker">un worker</param>
        /// <param name="fenetreParente">une fenêtre parente</param>
        public static void Afficher(string texte, Action worker, IWin32Window fenetreParente)
        {
            using (TraitementEnCoursForm traitementEnCoursForm = new TraitementEnCoursForm(worker))
            {
                traitementEnCoursForm.label.Text = texte;
                traitementEnCoursForm.StartPosition = FormStartPosition.CenterParent;
                traitementEnCoursForm.ShowDialog(fenetreParente);
            }
        }

        // ====================================================================================================
        // METHODES PROTEGEES
        // ====================================================================================================

        /// <summary>
        /// Chargement de la fenêtre
        /// </summary>
        /// <param name="e">un évènement</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            Task.Factory.StartNew(this.Worker).ContinueWith(t => { this.Close(); }, TaskScheduler.FromCurrentSynchronizationContext());
        }
    }
}
