﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SmartCowBioControl.src.objects
{
    public class BioControlPoids
    {
        // ====================================================================================================
        // PROPRIETES
        // ====================================================================================================

        /// <summary>
        /// Nb secondes.
        /// </summary>
        public int NbSecondes { get; private set; }

        /// <summary>
        /// Date et heure.
        /// </summary>
        public DateTime DateHeure { get; private set; }

        /// <summary>
        /// Numéro d'auge.
        /// </summary>
        public int NumAuge { get; private set; }

        /// <summary>
        /// Numéro d'animal.
        /// </summary>
        public int NumAnimal { get; private set; }

        /// <summary>
        /// Poids.
        /// </summary>
        public int Poids { get; private set; }

        /// <summary>
        /// Poids en kg.
        /// </summary>
        public double PoidsKg { get; private set; }

        /// <summary>
        /// PhotocellCovered (0 - not covered, 1- is covered).
        /// </summary>
        public int PhotocellCovered { get; private set; }

        /// <summary>
        /// GatePosition(0 - gate down, 1 - gate up).
        /// </summary>
        public int GatePosition { get; private set; }

        // ====================================================================================================
        // CONSTRUCTEURS
        // ====================================================================================================

        /// <summary>
        /// Constructeur.
        /// </summary>
        /// <param name="date">une date</param>
        /// <param name="nbSecondes">un nombre de secondes après minuit</param>
        /// <param name="numAuge">un numéro d'auge</param>
        /// <param name="numAnimal">un numéro d'animal</param>
        /// <param name="poids">un poids</param>
        /// <param name="photocellCovered">un indicateur de photocellule couverte</param>
        /// <param name="gatePosition">un indicateur de position de porte</param>
        public BioControlPoids(DateTime date, int nbSecondes, int numAuge, int numAnimal, int poids, int photocellCovered, int gatePosition)
        {
            this.NbSecondes = nbSecondes;
            this.DateHeure = date.Date.AddSeconds(nbSecondes);
            this.NumAuge = numAuge;
            this.NumAnimal = numAnimal;
            this.Poids = poids;
            this.PoidsKg = poids / 1000.0;
            this.PhotocellCovered = photocellCovered;
            this.GatePosition = gatePosition;
        }
    }
}
