﻿using System;

namespace SmartCowBioControl.src.objects
{
    /// <summary>
    /// Classe représentant une visite.
    /// </summary>
    public class BioControlVisite
    {
        // ====================================================================================================
        // PROPRIETES
        // ====================================================================================================

        /// <summary>
        /// Numéro d'animal.
        /// </summary>
        public int NumAnimal { get; private set; }

        /// <summary>
        /// Numéro d'auge.
        /// </summary>
        public int NumAuge { get; private set; }

        /// <summary>
        /// Numéro d'aliment.
        /// </summary>
        public string NumAliment { get; private set; }

        /// <summary>
        /// Date et heure de début.
        /// </summary>
        public DateTime DateHeureDebut { get; private set; }

        /// <summary>
        /// Date et heure de fin.
        /// </summary>
        public DateTime DateHeureFin { get; private set; }

        /// <summary>
        /// Durée.
        /// </summary>
        public int Duree { get; private set; }

        /// <summary>
        /// Quantité consommée.
        /// </summary>
        public int QuantiteConsommee { get; private set; }

        // ====================================================================================================
        // CONSTRUCTEURS
        // ====================================================================================================

        /// <summary>
        /// Constructeur.
        /// </summary>
        /// <param name="numAnimal">un numéro d'animal</param>
        /// <param name="numAuge">un numéro d'auge</param>
        /// <param name="numAliment">un numéro d'aliment</param>
        /// <param name="dateHeureFin">une date et heure de fin</param>
        /// <param name="duree">une durée</param>
        /// <param name="quantiteConsommee">une quantité consommée</param>
        public BioControlVisite(int numAnimal, int numAuge, string numAliment, DateTime dateHeureFin, int duree, int quantiteConsommee)
            : this(numAnimal, numAuge, numAliment, dateHeureFin, quantiteConsommee)
        {
            this.DateHeureDebut = dateHeureFin.AddSeconds(-duree);
            this.Duree = duree;
        }

        /// <summary>
        /// Constructeur.
        /// </summary>
        /// <param name="numAnimal">un numéro d'animal</param>
        /// <param name="numAuge">un numéro d'auge</param>
        /// <param name="numAliment">un numéro d'aliment</param>
        /// <param name="dateHeureDebut">une date et heure de début</param>
        /// <param name="dateHeureFin">une date et heure de fin</param>
        /// <param name="quantiteConsommee">une quantité consommée</param>
        public BioControlVisite(int numAnimal, int numAuge, string numAliment, DateTime dateHeureDebut, DateTime dateHeureFin, int quantiteConsommee)
            : this(numAnimal, numAuge, numAliment, dateHeureFin, quantiteConsommee)
        {
            this.DateHeureDebut = dateHeureDebut;
            this.Duree = (int)(dateHeureFin - dateHeureDebut).TotalSeconds;
        }

        /// <summary>
        /// Constructeur.
        /// </summary>
        /// <param name="numAnimal">un numéro d'animal</param>
        /// <param name="numAuge">un numéro d'auge</param>
        /// <param name="numAliment">un numéro d'aliment</param>
        /// <param name="dateHeureFin">une date et heure de fin</param>
        /// <param name="quantiteConsommee">une quantité consommée</param>
        private BioControlVisite(int numAnimal, int numAuge, string numAliment, DateTime dateHeureFin, int quantiteConsommee)
        {
            this.NumAnimal = numAnimal;
            this.NumAuge = numAuge;
            this.NumAliment = numAliment;
            this.DateHeureFin = dateHeureFin;
            this.QuantiteConsommee = quantiteConsommee;
        }
    }
}
