﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SmartCowBioControl.src.objects.fichiers
{
    /// <summary>
    /// Classe abstraite représentant un fichier.
    /// </summary>
    public abstract class AbstractFichier
    {
        // ====================================================================================================
        // ATTRIBUTS
        // ====================================================================================================

        /// <summary>
        /// Le dossier.
        /// </summary>
        protected string dossier;

        /// <summary>
        /// La date.
        /// </summary>
        protected DateTime date;

        /// <summary>
        /// Le chemin.
        /// </summary>
        protected string chemin;

        // ====================================================================================================
        // PROPRIETES
        // ====================================================================================================

        /// <summary>
        /// Le format de nom de fichier.
        /// </summary>
        protected abstract string FormatNom { get; }

        /// <summary>
        /// Le format de date dans le nom de fichier.
        /// </summary>
        protected abstract string FormatDate { get; }

        // ====================================================================================================
        // CONSTRUCTEURS
        // ====================================================================================================

        /// <summary>
        /// Constructeur.
        /// </summary>
        /// <param name="dossier">un dossier</param>
        /// <param name="date">une date</param>
        protected AbstractFichier(string dossier, DateTime date)
        {
            this.dossier = dossier;
            this.date = date;
            this.chemin = dossier + Path.DirectorySeparatorChar + String.Format(this.FormatNom, date.ToString(this.FormatDate));
        }
    }
}
