﻿using SmartCowBioControl.src.util;
using System;
using System.Collections.Generic;
using System.IO;

namespace SmartCowBioControl.src.objects.fichiers
{
    /// <summary>
    /// Classe représentant un fichier de log.
    /// </summary>
    public class FichierLog : AbstractFichier
    {
        // ====================================================================================================
        // CONSTANTES
        // ====================================================================================================

        public const string FORMAT_FICHIER = "BC??????.LOG";

        // ====================================================================================================
        // PROPRIETES
        // ====================================================================================================

        /// <summary>
        /// <see cref="AbstractFichier.FormatNom"/>
        /// </summary>
        protected override string FormatNom => "BC{0}.LOG";

        /// <summary>
        /// <see cref="AbstractFichier.FormatDate"/>
        /// </summary>
        protected override string FormatDate => "yyMMdd";

        // ====================================================================================================
        // CONSTRUCTEURS
        // ====================================================================================================

        /// <summary>
        /// Constructeur.
        /// </summary>
        /// <param name="dossier">un dossier</param>
        /// <param name="date">une date</param>
        public FichierLog(string dossier, DateTime date)
            : base(dossier, date)
        { }

        // ====================================================================================================
        // METHODES PUBLIQUES
        // ====================================================================================================

        /// <summary>
        /// Retourne les visites du fichier.
        /// </summary>
        /// <returns>la liste des visites</returns>
        public List<BioControlVisite> GetVisites()
        {
            List<BioControlVisite> visites = new List<BioControlVisite>();

            if (File.Exists(this.chemin))
            {
                foreach (string ligne in File.ReadAllLines(this.chemin))
                {
                    BioControlVisite visite;
                    if (this.TryParseLigneLog(ligne, out visite))
                    {
                        visites.Add(visite);
                    }
                }
            }

            return visites;
        }

        // ====================================================================================================
        // METHODES PRIVEES
        // ====================================================================================================

        /// <summary>
        /// Lit une ligne de fichier.
        /// </summary>
        /// <param name="ligne">une ligne</param>
        /// <param name="visite">une visite (sortie)</param>
        /// <returns><code>true</code> si la ligne est valide, <code>false</code> sinon</returns>
        private bool TryParseLigneLog(string ligne, out BioControlVisite visite)
        {
            bool ligneValide = false;
            visite = null;

            string[] valeursLigne = Utilitaire.SplitEspaces(ligne.Trim());

            if (valeursLigne.Length == 24)
            {
                int quantiteAutorisee = int.Parse(valeursLigne[10]);
                ligneValide = quantiteAutorisee != 100000;

                if (ligneValide)
                {
                    int numAnimal = int.Parse(valeursLigne[0]);
                    int numAuge = int.Parse(valeursLigne[2]);

                    int annee = int.Parse(valeursLigne[3]) + 2000;
                    int mois = int.Parse(valeursLigne[4]);
                    int jour = int.Parse(valeursLigne[5]);
                    int heure = int.Parse(valeursLigne[6]);
                    int minute = int.Parse(valeursLigne[7]);
                    int seconde = int.Parse(valeursLigne[8]);
                    DateTime dateHeureFinVisite = new DateTime(annee, mois, jour, heure, minute, seconde);

                    int numAliment = int.Parse(valeursLigne[9]);

                    int quantiteConsommee = int.Parse(valeursLigne[11]);
                    int duree = int.Parse(valeursLigne[21]);

                    visite = new BioControlVisite(numAnimal, numAuge, numAliment.ToString(), dateHeureFinVisite, duree, quantiteConsommee);
                }
            }

            return ligneValide;
        }
    }
}
