﻿using SmartCowBioControl.src.util;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SmartCowBioControl.src.objects.fichiers
{
    /// <summary>
    /// Classe représentant un fichier Rapid.
    /// </summary>
    public class FichierRapid : AbstractFichier
    {
        // ====================================================================================================
        // PROPRIETES
        // ====================================================================================================

        /// <summary>
        /// <see cref="AbstractFichier.FormatNom"/>
        /// </summary>
        protected override string FormatNom => "{0}.txt";

        /// <summary>
        /// <see cref="AbstractFichier.FormatDate"/>
        /// </summary>
        protected override string FormatDate => "yyMMdd";

        // ====================================================================================================
        // CONSTRUCTEURS
        // ====================================================================================================

        /// <summary>
        /// Constructeur.
        /// </summary>
        /// <param name="dossier">un dossier</param>
        /// <param name="date">une date</param>
        public FichierRapid(string dossier, DateTime date)
            : base(dossier, date)
        { }

        // ====================================================================================================
        // METHODES PUBLIQUES
        // ====================================================================================================

        /// <summary>
        /// Retourne les poids du fichier.
        /// </summary>
        /// <returns>la liste des poids</returns>
        public List<BioControlPoids> GetPoids()
        {
            List<BioControlPoids> listePoids = new List<BioControlPoids>();

            if (File.Exists(this.chemin))
            {
                foreach (string ligne in File.ReadAllLines(this.chemin))
                {
                    BioControlPoids poids;
                    if (this.TryParseLigneRapid(ligne, out poids))
                    {
                        listePoids.Add(poids);
                    }
                }
            }

            return listePoids;
        }

        // ====================================================================================================
        // METHODES PRIVEES
        // ====================================================================================================

        /// <summary>
        /// Lit une ligne de fichier.
        /// </summary>
        /// <param name="ligne">une ligne</param>
        /// <param name="poids">un poids (sortie)</param>
        /// <returns><code>true</code> si la ligne est valide, <code>false</code> sinon</returns>
        private bool TryParseLigneRapid(string ligne, out BioControlPoids poids)
        {
            bool ligneValide;
            poids = null;

            string[] valeursLigne = Utilitaire.SplitEspaces(ligne.Trim());

            if (ligneValide = (valeursLigne.Length == 16))
            {
                int nbSecondes = int.Parse(valeursLigne[0]);
                int numAuge = int.Parse(valeursLigne[1]);
                int numAnimal = int.Parse(valeursLigne[2]);
                int valeurPoids = int.Parse(valeursLigne[3]);
                int photocellCovered = int.Parse(valeursLigne[10]);
                int gatePosition = int.Parse(valeursLigne[11]);

                poids = new BioControlPoids(this.date, nbSecondes, numAuge, numAnimal, valeurPoids, photocellCovered, gatePosition);
            }

            return ligneValide;
        }
    }
}
