﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace SmartCowBioControl.src.util
{
    /// <summary>
    /// Classe pour l'affichage des messages.
    /// </summary>
    public class AfficheurMessage
    {
        // ====================================================================================================
        // SINGLETON
        // ====================================================================================================

        /// <summary>
        /// Constructeur privé
        /// </summary>
        private AfficheurMessage() { }

        /// <summary>
        /// Holder
        /// </summary>
        private static class SingletonHolder
        {
            /// <summary>
            /// Instance unique non préinitialisée
            /// </summary>
            public static AfficheurMessage instance = new AfficheurMessage();
        }

        /// <summary>
        /// Point d'accès pour l'instance unique du singleton
        /// </summary>
        public static AfficheurMessage Instance { get { return SingletonHolder.instance; } }

        // ====================================================================================================
        // METHODES PUBLIQUES
        // ====================================================================================================

        /// <summary>
        /// Afficher un message d'erreur.
        /// </summary>
        /// <param name="message">un message d'erreur</param>
        public void AfficherMessageErreur(string message)
        {
            MessageBox.Show(message, "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        /// <summary>
        /// Afficher un message d'information.
        /// </summary>
        /// <param name="message">un message d'information</param>
        public void AfficherMessageInformation(string message)
        {
            MessageBox.Show(message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        /// <summary>
        /// Afficher un message d'avertissement.
        /// </summary>
        /// <param name="message">un message d'avertissement</param>
        public void AfficherMessageAvertissement(string message)
        {
            MessageBox.Show(message, "Avertissement", MessageBoxButtons.OK, MessageBoxIcon.Warning);
        }

        /// <summary>
        /// Afficher un message d'avertissement avec choix.
        /// </summary>
        /// <param name="message">un message d'avertissement</param>
        /// <param name="boutons">des boutons ("OK / Annuler" par défaut)</param>
        public DialogResult AfficherMessageAvertissementChoix(string message, MessageBoxButtons boutons = MessageBoxButtons.OKCancel)
        {
            return MessageBox.Show(message, "Avertissement", boutons, MessageBoxIcon.Warning);
        }

        /// <summary>
        /// Afficher des messages d'avertissement avec choix.
        /// </summary>
        /// <param name="messages">des messages d'avertissement</param>
        /// <param name="boutons">des boutons ("OK / Annuler" par défaut)</param>
        public DialogResult AfficherMessageAvertissementChoix(IEnumerable<string> messages, MessageBoxButtons boutons = MessageBoxButtons.OKCancel)
        {
            return this.AfficherMessageAvertissementChoix(this.GetMessage(messages), boutons);
        }

        // ====================================================================================================
        // METHODES PRIVEES
        // ====================================================================================================

        /// <summary>
        /// Retourne un message à partir d'une liste.
        /// </summary>
        /// <param name="messages">des messages</param>
        /// <returns>un message</returns>
        private string GetMessage(IEnumerable<string> messages)
        {
            return string.Join(Environment.NewLine, messages);
        }
    }
}
