﻿using System.ComponentModel;

namespace SmartCowBioControl.src.util
{
    /// <summary>
    /// Paramètres des reports de progression (pour un traitement batch).
    /// </summary>
    public class ReportProgressParameters
    {
        // ====================================================================================================
        // ATTRIBUTS
        // ====================================================================================================

        /// <summary>
        /// Le pourcentage de progression.
        /// </summary>
        private int progressPercentage = 0;

        /// <summary>
        /// Le nombre d'éléments traités.
        /// </summary>
        private int nbElementsTraites = 0;

        /// <summary>
        /// Le nombre d'unités traitées.
        /// </summary>
        private long nbUnitesTraites = 0;

        /// <summary>
        /// Le nombre d'erreurs.
        /// </summary>
        private int nbErreurs = 0;

        // ====================================================================================================
        // PROPRIETES
        // ====================================================================================================

        /// <summary>
        /// Le BackgroundWorker.
        /// </summary>
        public BackgroundWorker BackgroundWorker { get; private set; }

        // ====================================================================================================
        // CONSTRUCTEURS
        // ====================================================================================================

        /// <summary>
        /// Constructeur.
        /// </summary>
        /// <param name="backgroundWorker">le BackgroundWorker</param>
        public ReportProgressParameters(BackgroundWorker backgroundWorker)
        {
            this.BackgroundWorker = backgroundWorker;
        }

        // ====================================================================================================
        // METHODES PUBLIQUES
        // ====================================================================================================

        /// <summary>
        /// Augmente le pourcentage de progression.
        /// </summary>
        /// <param name="nbPercentage">le nombre de pourcentage à augmenter (1 par défaut)</param>
        public void AddPercentage(int nbPercentage = 1)
        {
            this.progressPercentage += nbPercentage;
        }

        /// <summary>
        /// Indique que un élément est traité.
        /// </summary>
        /// <param name="etapesAAjouter">un nombre d'étapes à ajouter</param>
        /// <param name="nbUnites">le nombre d'unités de l'élément</param>
        public void ElementTraite(int etapesAAjouter, long nbUnites, bool isErreur)
        {
            this.progressPercentage += etapesAAjouter;
            this.nbElementsTraites++;
            this.nbUnitesTraites += nbUnites;
            if (isErreur) this.nbErreurs++;
        }

        /// <summary>
        /// Reporte une progression (pour un traitement batch).
        /// </summary>
        /// <param name="parameters">les paramètres de progression</param>
        public static void ReportProgress(object parameters)
        {
            ReportProgressParameters reportProgressParameters = parameters as ReportProgressParameters;

            if (reportProgressParameters != null && reportProgressParameters.BackgroundWorker != null && reportProgressParameters.BackgroundWorker.IsBusy)
            {
                reportProgressParameters.BackgroundWorker.ReportProgress(
                    reportProgressParameters.progressPercentage,
                    new long[] {
                        reportProgressParameters.nbElementsTraites,
                        reportProgressParameters.nbUnitesTraites,
                        reportProgressParameters.nbErreurs});
            }
        }
    }
}
